/*
 * @(#)events.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 * Java Virtual Machine Event Interface (JVMEI)
 *
 * Public structures and interfaces
 *
 */

#ifndef _EVENTS_H
#define _EVENTS_H

#include "jni.h"

/* Event IDs */
#define JVMEI_EVENT_HANDLER_REGISTERED    1
#define JVMEI_EVENT_HANDLER_DEREGISTERED  2
#define JVMEI_EVENT_THREAD_CREATE         3
#define JVMEI_EVENT_THREAD_START          4
#define JVMEI_EVENT_THREAD_END            5
#define JVMEI_EVENT_THREAD_ATTACH         6                        /*ibm.8601*/
#define JVMEI_EVENT_THREAD_DETACH         7                        /*ibm.8601*/
#define JVMEI_EVENT_MAX                   7                        /*ibm.8601*/

/* Special token values */
#define JVMEI_TOKEN_SYSTEM                0
#define JVMEI_TOKEN_INVALID              -1

typedef jint jtoken;

/* Event data structure */
struct JVMEI_Event {
    jtoken  eventToken;
    jint    eventType;
    jtoken  threadToken;
    JNIEnv *env;
    void   *pHandlerGlobalStorage;
    union {
        struct {
            jtoken newThreadToken;
        } thread_create;
        struct {
            void *pEventData;
        } user_event;
    } eu;
};
typedef struct JVMEI_Event JVMEI_Event_t;

/* Function prototypes */
jtoken JEI_RegisterEventHandler(void (*pEventHandler)(JVMEI_Event_t *pEventData),
                                void *pHandlerGlobalStorage);
jint   JEI_DeregisterEventHandler(jtoken handlerToken);
jint   JEI_EnableEvent(jtoken handlerToken, jint eventType);
jint   JEI_DisableEvent(jtoken handlerToken, jint eventType);
void * JEI_GetThreadLocalStorage(jtoken handlerToken, jtoken threadToken);
jint   JEI_SetThreadLocalStorage(jtoken handlerToken, jtoken threadToken,
                                 void *pThreadStorage);
jint   JEI_NotifyEvent(jtoken handlerToken, jint eventType, void *pEventData);

#endif /* !_EVENTS_H */
