/*
 * @(#)finalize.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/*
 * @(#)finalize.h       1.19 97/02/21
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 */

#ifndef _FINALIZE_H_
#define _FINALIZE_H_

#include "oobj.h"
#include "sys_api.h"

#if defined(IBM_OE_THREADS)                                         /*ibm.3286*/
#include "monitor_md.h"
#endif                                                              /*ibm.3286*/

/*
 * The HasFinalizerQ and FinalizeMeQ queues contain finalizer_t
 * structures.  Handles rather than objects are used to avoid relocating
 * the contents of the queues on GC.
 */
typedef struct {
    JHandle *next;              /* The next finalizer object */
} finalizer_t;

extern JHandle *HasFinalizerQ;
extern JHandle *FinalizeMeQ;
extern JHandle *FinalizeMeQLast;
extern bool_t   FinalizedNotCollected;				/*ibm.12268*/
// ibm.9026
#ifdef IBM_HAIFA_GC
extern JHandle *BeingFinalized;
#endif //IBM_HAIFA_GC 
// ibm.9026 ends


extern bool_t finalize_on_exit;

#ifdef JMS_TRL                                                          /*ibm*/
#define FINALIZER_T(h) \
        ((finalizer_t *)((char *)unhand(h) + cbInstanceSize(obj_classblock(h))))
#else                                                                   /*ibm*/
#define FINALIZER_T(h) \
        ((finalizer_t *)((char *)h->obj + cbInstanceSize(obj_classblock(h))))
#endif                                                                  /*ibm*/

/*
 * Locks for the finalization queues
 */
extern sys_mon_t *_hasfinalq_lock;
#define HASFINALQ_LOCK_INIT()   monitorRegister(_hasfinalq_lock, \
                                                "Has finalization queue lock")
#define HASFINALQ_LOCK()        sysMonitorEnter(_hasfinalq_lock)
#define HASFINALQ_LOCKED()      sysMonitorEntered(_hasfinalq_lock)
#define HASFINALQ_UNLOCK()      sysMonitorExit(_hasfinalq_lock)
#define HASFINALQ_NOTIFY()      sysMonitorNotify(_hasfinalq_lock)

extern sys_mon_t *_finalmeq_lock;
#define FINALMEQ_LOCK_INIT()    monitorRegister(_finalmeq_lock, \
                                                "Finalize me queue lock")
#define FINALMEQ_LOCK()         sysMonitorEnter(_finalmeq_lock)
#define FINALMEQ_LOCKED()       sysMonitorEntered(_finalmeq_lock)
#define FINALMEQ_UNLOCK()       sysMonitorExit(_finalmeq_lock)
#define FINALMEQ_NOTIFY()       sysMonitorNotify(_finalmeq_lock)
#define FINALMEQ_WAIT()         sysMonitorWait(_finalmeq_lock, \
                                               SYS_TIMEOUT_INFINITY, FALSE)

extern void InitializeFinalizer(void);
extern void InitializeFinalizerThread(void);

extern void runFinalization(void);
extern void finalizeOnExit(void);

#endif /* _FINALIZE_H_ */
