/*
 * @(#)gc.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/*
 * @(#)gc.h     1.10 97/01/24
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 */

#ifndef _GC_H_
#define _GC_H_

#ifdef IBM_FASTSWEEP							/*ibm.6833*/
#define DENSE_MARKBITS
#endif									/*ibm.6833*/
#if defined(IBM_INTEL)							/*ibm.8904*/
#define COMPACTION_AVOIDANCE
#endif									/*ibm.8904*/

#include "gc_md.h"
#ifdef IBM_HAIFA_GC
#include "threads.h"
#include "interpreter.h"
#include "threads_md.h"
#include "gc_protocol.h"
#endif /* IBM_HAIFA_GC */

#ifdef JMS_TRL                                                          /*ibm*/
#if defined(IBM_OS2)							/*ibm.3243*/
#include "mutex_md.h"
#include "finalize.h"							/*ibm.3363*/
#endif									/*ibm.3243*/
#define HEAP_LOCK_INIT() sysHeapLockInit()
#define HEAP_LOCK()      sysHeapLock()
#define HEAP_UNLOCK()    sysHeapUnlock()
#define HEAP_LOCKED()    sysHeapLocked()
/* [TRL] Fastpath */
#ifndef IBM_HAIFA_GC
typedef long hdr;
typedef struct chunk {
    hdr      header;
    struct chunk* next;
} Chunk;
#define MAX_FREELIST 64
#define OBJECTGRAIN     8
#define LOG_OBJECTGRAIN 3
typedef long hhdr;
#define OLINK_SIZE  sizeof(hhdr)
#define HEADER_SIZE  (OLINK_SIZE+HANDLE_SIZE)

/* mark address offset from handle */
#define MARK_OFFSET  0
#define _ALLOCED_  1
#endif /* not IBM_HAIFA_GC */

#define HASHCORE_INCREMENT  4
#if defined(IBM_FASTNEW)
typedef struct {
#if defined(IBM_AIX) || defined(IBM_4690) || defined(IBM_LINUX) /*ibm.7306*/
  sys_mon_t *      p_heap_mon;
#else                                                                   /*ibm*/
  mutex_t *        p_heap_lock;
#endif                                                                  /*ibm*/
  unsigned char ** p_heapbase;
  unsigned char ** p_heaplimit;
#ifdef COMPACTION_AVOIDANCE 						/*ibm.8904*/
  long *           p_allocLocalSize;
#else									/*ibm.8904*/
  Chunk **         p_freelist;
#endif /* COMPACTION_AVOIDANCE */					/*ibm.8904*/
  int  *           p_heap_memory_changes;
  long *           p_FreeObjectCtr;
  unsigned int **  p_allocbits;
  long *           p_current_hashcore;
  sys_mon_t **     p_hasfinalq_lock;					/*ibm.3363*/
  JHandle **       p_HasFinalizerQ;					/*ibm.3363*/
#if defined(IBM_ALLOC_CACHE)						/*ibm.4816*/
  HObject *        (*p_FastObjAlloc)(ExecEnv*, ClassClass*, long);
  HObject *        (*p_FastArrayAlloc)(ExecEnv*, int, int);
  HObject *        (*p_cacheAlloc)(ExecEnv*, struct methodtable*, long, int);
#endif									/*ibm.4816*/
} GCLinkVector;

extern GCLinkVector p_GCLinkVector;
#endif									/*ibm.3105*/
#else                                                                   /*ibm*/
/*
 * Lock against heap modification.
 */
#if defined(IBM_OS2)                                                    /*ibm*/
#include <javaos2.h>
#include <mutex_md.h>

extern mutex_t _heap_lock;
#define HEAP_LOCK_INIT() mutexInit(&_heap_lock)
#define HEAP_LOCK() mutexLock(&_heap_lock)
#define HEAP_UNLOCK() mutexUnlock(&_heap_lock)
#define HEAP_LOCKED() mutexLocked(&_heap_lock)
#else                                                                   /*ibm*/
extern sys_mon_t *_heap_lock;
#define HEAP_LOCK_INIT()    monitorRegister(_heap_lock, "Heap lock")
#define HEAP_LOCK()         sysMonitorEnter(_heap_lock)
#define HEAP_UNLOCK()       sysMonitorExit(_heap_lock)
#define HEAP_LOCKED()       sysMonitorEntered( _heap_lock)
#endif                                                                  /*ibm*/

/*
 * Define this if you want the mark phase to detect pointers into the
 * interior of objects.
 */
/* #define CHECK_INTERIOR_POINTERS */

#define OBJECTGRAIN     8
#define HANDLEGRAIN     8
#define BITSPERCHAR     8

/*
 * Types of overflows: we might respond to an overflow of a particular
 * error differently, e.g. expanding only the overflowing area.
 */
#define OVERFLOW_NONE    0
#define OVERFLOW_OBJECTS 1
#define OVERFLOW_HANDLES 2

/*
 * Possible actions to take on overflows.  manageAllocFailure()
 * decides between these.
 */
#define OVERFLOW_ACT_FAIL       0
#define OVERFLOW_ACT_GC         1
#define OVERFLOW_ACT_FINALIZE   2
#define OVERFLOW_ACT_REFS       3
#define OVERFLOW_ACT_EXPAND     4
#define OVERFLOW_ACT_DESPERATE  5

/*
 * Memory block header (bottom three bits are flags):
 *
 * -------------------------------------------------------------
 * | <--- length --->| pinned | <- obj swapped -> | <- free -> |
 * -------------------------------------------------------------
 * 31                3        2                   1            0
 */
typedef long hdr;

#define obj_geth(p) (*((hdr *)(p)))
#define obj_seth(p, h) (*((hdr *)(p)) = (h))
#define h_len(h) ((h) & ~(OBJECTGRAIN-1))
#define h_free(h) ((h) & 1)
#define h_bumplen(h, l) ((h) += (l))

#define obj_len(p) (obj_geth(p)&~(OBJECTGRAIN-1))
#define obj_setlf(p, l, f) (obj_geth(p) = (l)|(f))
#define obj_bumplen(p, l) (obj_geth(p) += (l))
#define obj_free(p) (obj_geth(p)&1)
#define obj_setfree(p) (obj_geth(p) |= 1)
#define obj_clearfree(p) (obj_geth(p) &= ~1)
#define obj_pinned(p) (obj_geth(p) & 4)
#define obj_pin(p) (obj_geth(p) |= 4)
#define obj_unpin(p) (obj_geth(p) &= ~4)

/*
 * The marking code relies upon the values representing the three mark
 * states to be ordered numerically: NoMark < SoftMark < HardMark.
 */
#define NoMark   0
#define SoftMark 1
#define HardMark 3

#define MarkPtr(p, v) _MarkPtr(((unsigned int) (p) & ~(OBJECTGRAIN - 1)), v)
#define ClearMarkPtr(p, v) _ClearMarkPtr(((unsigned int)(p)&~(OBJECTGRAIN-1)),v)
#define IsMarked(p) _IsMarked((unsigned int) (p) & ~(OBJECTGRAIN - 1))

#define SOFTREFBAGSIZE 200  /* max number of soft refs to kill in one cycle */


#ifndef PAGED_HEAPS /************ CONTIGUOUS HEAPS: ********************/

#define ValidObject(p)  ((((int)(p)) & (OBJECTGRAIN-1)) == 0 &&         \
                         (unsigned char *)(p) >= opmin &&               \
                         (unsigned char *)(p) <  opmax)
#define ValidHandle(p)  (((int) (p) & (sizeof(JHandle)-1)) == 0 &&      \
                         (unsigned char *)(p) >= hpmin &&               \
                         (unsigned char *)(p) <= hpmax)
/* ValidHorO() assumes OBJECTGRAIN=sizeof(JHandle)... */
#define ValidHorO(p)    (((int) (p) & (OBJECTGRAIN-1)) == 0 &&          \
                         (unsigned char *)(p) >= hpmin &&               \
                         (unsigned char *)(p) <= opmax)
#define SetLimits()                                                     \
    register unsigned char *const opmin = opool,                        \
                           *const opmax = opoollimit,                   \
                           *const hpmin = hpool,                        \
                           *const hpmax = hpoollimit-sizeof(JHandle)

#define POP_FREE_HANDLE(hp)                                             \
    hp = (JHandle *)hpoolfreelist;                                      \
    if (hp) {                                                           \
        hpoolfreelist = (unsigned char *)hp->methods;                   \
    }

#define PUSH_FREE_HANDLE(hp) \
    hp->methods = (struct methodtable *)hpoolfreelist; \
    hpoolfreelist = (unsigned char *)hp;

/* Mark bit access assumes contiguity of handles and objects */
#define MARKINDEX(p)    (((unsigned char *)(p) - hpmin) >> 7)
#define BITOFFSET(p)    ((((unsigned char *)(p) - hpmin) >> 2) & 0x1e)

#define _MarkPtr(p, v)  (markbits[MARKINDEX(p)] |= (v) << BITOFFSET(p))
#define _ClearMarkPtr(p, v) (markbits[MARKINDEX(p)] &= ~((v) << BITOFFSET(p)))
#define _IsMarked(p)    ((markbits[MARKINDEX(p)] >> BITOFFSET(p)) &3)

/* set the second word in an object (from ptr to header) to 0x55555555 */
#define CHECK_WORD_INDEX 1

#define MAP_OVER_HANDLES_FROM_START(MO_hp) {            \
    JHandle *MOH_limit = (JHandle *) hpmax;             \
    for (MO_hp = (JHandle *) hpool; MO_hp <= MOH_limit; MO_hp++) {

#define END_MAP_OVER_HANDLES_FROM_START                 \
    } /* end for */                                     \
} /* end MAP_OVER_HANDLES_FROM_START */

#define MAP_OVER_OBJECTS_FROM_START(p) {                \
    unsigned char *MOO_limit = opmax;                   \
    unsigned char *MOO_start = opmin;                   \
    for (p = opmin;                                     \
         p < MOO_limit;                                 \
         p += obj_len(p)) {

#define END_MAP_OVER_OBJECTS_FROM_START                 \
    } /* end for */                                     \
} /* end END_MAP_OVER_OBJECTS_FROM_START */


#else /************ PAGED HEAPS: ********************/

/* gc philosophy makes it necessary to detect if an arbitrary int is
 * (possibly) a handle or object ref.
 * A value is (possibly) valid if it is properly aligned, and it
 * points into a page that has a page map entry of the proper type.
 */
/* assumes ValidHorO already */
#define GetPageMapEntry(p)                                              \
             (page_map[((int)(p) - (int)mem_base) >> PTR_2_PAGE_SHIFT])

#define ValidObject(p)  ((((int)(p)) & (OBJECTGRAIN-1)) == 0 &&         \
             (void *)(p) >= mem_base &&                                 \
             (void *)(p) <  mem_top &&                                  \
             (GetPageMapEntry((p)).chunk_size > 0))
#define ValidHandle(p)  (((((int)(p)) & (HANDLEGRAIN-1)) == 0) &&       \
             ((void *)(p) >= mem_base) &&                               \
             ((void *)(p) <  mem_top) &&                                \
             (GetPageMapEntry((p)).chunk_size < 0))
/* ValidHorO() assumes OBJECTGRAIN == HANDLEGRAIN... */
#define ValidHorO(p)    ((((int)(p)) & (HANDLEGRAIN-1)) == 0 &&         \
             (void *)(p) >= mem_base &&                                 \
             (void *)(p) <  mem_top &&                                  \
             (GetPageMapEntry((p)).chunk_size != 0))

#define SetLimits() int SL_dufus = 0

/* assumes ValidHorO already */
#define ChunkBase(p)    (void *)                                        \
             (((int)(p) & ~(PAGE_ALIGNMENT - 1)) -                      \
              (GetPageMapEntry((p)).page_number << PTR_2_PAGE_SHIFT))

/* curHanBlkP must be set in advance!!! */
#define POP_FREE_HANDLE(hp)                                             \
    hp = (JHandle *)curHanBlkP->freePtr;                                \
    if (hp) {                                                           \
        curHanBlkP->freePtr = (unsigned char *)hp->methods;             \
    }

/* Can only be called within a MAP_OVER_HANDLES_FROM_START loop
 * - uses MOH_chunk instead of curHanBlkP for efficiency.
 */
#define PUSH_FREE_HANDLE(hp) \
    hp->methods = (struct methodtable *)MOH_chunk->freePtr; \
    MOH_chunk->freePtr = (unsigned char *)hp;

#define MARKINDEX(p)    (((int)(p) & (PAGE_ALIGNMENT - 1)) >> 7)
#define BITOFFSET(p)    ((((int)(p) & (PAGE_ALIGNMENT - 1)) >> 2) & 0x1e)

#define _MarkPtr(p, v)  (GetPageMapEntry(p).mark_bits[MARKINDEX(p)] |=     \
                         (v) << BITOFFSET(p))
#define _ClearMarkPtr(p, v) (GetPageMapEntry(p).mark_bits[MARKINDEX(p)] &= \
                             ~((v) << BITOFFSET(p)))
#define _IsMarked(p)    ((GetPageMapEntry(p).mark_bits[MARKINDEX(p)]       \
                          >> BITOFFSET(p)) & 3)

/* # of bytes of markbits we need per page: */
#define MARK_BITS_SIZE      ((PAGE_ALIGNMENT / (OBJECTGRAIN * BITSPERCHAR)) * 2)

/*
 * Part of Java memory management and garbage collection.
 *
 * This supports a discontiguous gcable heap, which is useful for the
 * Mac OS, or other platforms without good memory mapping support.
 *
 * CHUNKS:
 * Memory is requested from the OS in "Chunks" of n pages, which are
 * PAGE_ALIGNMENT aligned and sized. Handles and objects are allocated out of
 * different chunks.  When more memory is needed, additional chunks can be
 * allocated.  When chunks are free, they may be returned to  the OS.  Chunks
 * don't need to be contiguous.  Handle chunks and object chunks are linked
 * into seperate, doubly linked lists, which are sorted by chunk address.  On
 * platforms without real "memalign" support, there may be unaligned (wasted)
 * space that precedes the true chunk that we can use for something else
 * (markbits come to mind).
 */

/* fields marked ### MUST BE OBJECT AND/OR HANDLE GRAIN ALIGNED */
typedef struct CHUNK_BLK {      /* a chunk of pages */
    void* chunkHandle;          /* OS handle to this chunk */
    struct CHUNK_BLK *nextPtr;  /* ptr to next chunk header */
    struct CHUNK_BLK *prevPtr;  /* ptr to previous chunk header */
    long chunkFlags;            /* misc flags */
    long allocSize;             /* == (endPtr - startPtr)### */
    long freeCnt;               /* # of free bytes in this chunk */
    unsigned char *startPtr;    /* ptr to starting byte### */
    unsigned char *endPtr;      /* ptr past last byte### */
    unsigned char *freePtr;     /* ptr to first free space CANDIDATE
        * (may not really be free), or it might be a ptr to a free list
        * of objects, depending on phase of the moon.
        */
        /* users may modify start and end ptrs, but not this one: */
    unsigned char *physEndPtr;
#ifdef WASTED_SPACE_IN_LEADER
        /* WARNING:  clearLocalMarkBits assumes that only markbits are stored
         * in the waste !!!
         */
    unsigned char *wasteStartPtr; /* ptr to starting wasted byte */
    unsigned char *wasteFreePtr;  /* ptr to first free wasted byte */
                                  /* wasteEndPtr == the ChunkBlk pointer */
#endif /* WASTED_SPACE_IN_LEADER*/
} ChunkBlk, *ChunkBlkP;

/* CHUNK_BLK->chunkFlags bits: */
     /* set this bit in chunkFlags if any objects in the chunk are pinned */
#define CHUNK_PINNED 1

/* doubly-linked list of handle chunks, in address order: */
extern ChunkBlkP firstHanBlkP;
extern ChunkBlkP lastHanBlkP;
extern ChunkBlkP curHanBlkP;
/* doubly-linked list of object chunks, in address order: */
extern ChunkBlkP firstObjBlkP;
extern ChunkBlkP lastObjBlkP;
extern ChunkBlkP curObjBlkP;

/* store this into the last two words of the chunk to detect overwrites */
/* Odd to make a poor pointer, 111 in the low bits looks like a swapped
 * free block if a header! */
#define ALMOST_WORD    0x77777777
#define ULTIMATE_WORD  0xBAADDEED  /* Why not. */
/* set the third word in an object (from ptr to header) to 0x55555555 */
#define CHECK_WORD_INDEX 2

/* Macros to abstract out looping over all handles or objects.
 * Note that you can't "break" out of this loop. Use goto or return instead.
 */
#define MAP_OVER_HANDLES_FROM_START(MO_hp) {            \
    ChunkBlkP MOH_chunk = firstHanBlkP;                 \
    JHandle *MOH_limit;                                 \
    do {                                                \
        for (MO_hp = (JHandle *)MOH_chunk->startPtr,    \
             MOH_limit = (JHandle *)MOH_chunk->endPtr;  \
             MO_hp < MOH_limit; MO_hp++) {

#define END_MAP_OVER_HANDLES_FROM_START                 \
        }  /* end for */                                \
        MOH_chunk = MOH_chunk->nextPtr;                 \
    } while (MOH_chunk != firstHanBlkP);                \
} /* end MAP_OVER_HANDLES_FROM_START */

#define MAP_OVER_OBJECTS_FROM_START(MO_p)   {           \
    ChunkBlkP MOO_chunk = firstObjBlkP;                 \
    unsigned char *MOO_limit;                           \
    unsigned char *MOO_start;                           \
    do {                                                \
        for ((MO_p) = MOO_chunk->startPtr,              \
             MOO_start = MOO_chunk->startPtr,           \
             MOO_limit = MOO_chunk->endPtr;             \
             (MO_p) < MOO_limit;                        \
             (MO_p) += obj_len(MO_p)) {

#define END_MAP_OVER_OBJECTS_FROM_START                 \
        }  /* end for */                                \
        MOO_chunk = MOO_chunk->nextPtr;                 \
    } while (MOO_chunk != firstObjBlkP);                \
} /* end END_MAP_OVER_OBJECTS_FROM_START */

#endif /************ END PAGED HEAPS ********************/


#ifdef WASTED_SPACE_IN_LEADER
/* following functions defined in gc_md.c: */
void initWastedSpaceInChunk(ChunkBlkP chunk);
void sysCheckWastedSpace(ChunkBlkP chunk);
void clearLocalMarkBits(void);
void* allocMarkBitsLocally(ChunkBlkP blkP);
#else
#define initWastedSpaceInChunk(xxx) 0
#define sysCheckWastedSpace(xxx) 0
#define clearLocalMarkBits() 0
#define allocMarkBitsLocally(xxx) 0
#endif

#endif

#ifdef IBM_HAIFA_GC

/**********************************************************************
 *
 * FUNCTION NAME: markBlack
 * Description: Finds the object's sons (scanHandle) and mark them gray
 *              and then mark the object Black
 *
 * INPUT: JHandle * h - a pointer to the object on the heap.
 * OUTPUT: The object turn black and its sons gray.
 * RETURN: None
 *
 **********************************************************************/
#define markBlack(h)                          \
{                                             \
  if (!(GET_COLOR(hand2mark(h))==BLACK)) {    \
    scanHandle(h);                            \
    SET_COLOR(hand2mark(h),BLACK);            \
  }                                           \
}

/**********************************************************************
 *
 * FUNCTION NAME: postHandshake
 * Description: Sets the global collector status which should be checked by
 *              the mutators.
 * INPUT: status - the status to change to ( SYNC1, SYNC2, ASYNC )
 * OUTPUT: GCstatus
 * RETURN: None
 *
 **********************************************************************/
#define postHandshake(stat) (GCstatus = stat)

/* Tuning parameters */
#define WAITTIME             1      /* The total time to wait until sending interupt */
#define MAXWAITTOFIN         500      /* if there is more then MAXWAITTOFIN objects
				       waiting for finalization run sync finalization                                        from gc treade after async gc call */
#define MIN_FREE_BYTES 	((int)(TotalObjectCtr*0.5))
#define HIGHWATER_MARK ((char *)(TotalObjectCtr*0.8))
#define THRESHOLD_ADD 0.2*iFreeBytes
#define THRESHOLD_MAX ((int)(TotalObjectCtr*0.9))
extern  int MaxAllocToGC, MaxAllocToFullGC;                /*ibm.5833*/

#define ALLOC_TO_PART_COLLECT     1000000                  /*ibm.5833*/
#define ALLOC_TO_FULL_COLLECT     ((int)(TotalObjectCtr*0.5))    /*ibm.5833*/
#endif /* IBM_HAIFA_GC */

#ifdef DENSE_MARKBITS		/*ibm.6833*/
typedef struct {          	/*ibm.6833*/
    JHandle *next;         	/*ibm.6833 The next SoftRef object */
} softref_t;		        /*ibm.6833*/
#endif 				/*ibm.6833*/
                                                           /*ibm*/
#endif /* !_GC_H_ */
