/*
 * @(#)heapmanager.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
#ifndef _HM
#define _HM
#include "sys_api.h"         /* for sysAssert */
#include "gc_protocol.h"
#include "finalize.h" 

/* --------- Convenience functions for accessing structures ------------- */
/* Large Objects */
//#define RegionSize(ud)   ((ud)->iRegionSize & ~0x03)
#define RegionSize(ud)   ((((ud)->iRegionSize & ~0x03)<BLOCKSIZE) ? BLOCKSIZE \
                         : ((ud)->iRegionSize & ~0x03) )
#define LeftChild(ud)    (ud)->u.udLargeObj.pLeftChild
#define RightChild(ud)   (ud)->u.udLargeObj.pRightChild
#define LLHeadOrNext(ud) (ud)->u.udLargeObj.pLLHeadOrNext
#define LLParent(ud)     RightChild(ud)
#define LLNext(ud)       LLHeadOrNext(ud)

/* Small Objects */
#define ChunkSize(ud)    ((ud)->iRegionSize & ~0x03)
#define PrevBlock(ud)    (ud)->u.udSmallObj.pPrevBlock
#define NextBlock(ud)    (ud)->u.udSmallObj.pNextBlock

#define NextChunk(iAddress) (* (long *) iAddress)
#if 0
#define SetChunkSize(ud, iSize) (ud)->iRegionSize = iSize
#endif
/* ibm.6790 */
#define SetChunkSize(ud, iSize) { \
int iSetBits=((long)(ud)->iRegionSize & 1) | iSize; \
(ud)->iRegionSize = iSetBits;  \
}

/* Write the region size into both the first and last header block associated
   with an object */
#define NUM_HDR_BLKS(iSize) ((int)((iSize + BLOCKSIZE - 1)/BLOCKSIZE))

#define GET_LAST_HDR_BLK(p, iSize) ((BlockStructPtr)((char *) p + \
                               sizeof(blockStruct)*(NUM_HDR_BLKS(iSize) - 1)))

/* define SetRegionSize(ud, iSize) (ud)->iRegionSize = iSize;*/
#define SetRegionSize(ud, iSize) { \
int iSetBits=((long)(ud)->iRegionSize & 3) | iSize; \
BlockStructPtr pLastHdr = GET_LAST_HDR_BLK((ud), iSize); \
(ud)->iRegionSize = iSetBits;  \
if (iSize > BLOCKSIZE) \
pLastHdr->iRegionSize = iSetBits; \
}


/******* Take this out on integration !!!!!! (already in gc_ibm.c) ***********/

/* Common to both small and large objects */
/* Allocated bit is LSB.  Partially filled bit is 2d bit */
#define GetAllocated(ptr)   ((ptr)->iRegionSize & 0x00000001)
/* For SetAllocated and ClearAllocated, update flags in the last header blk
   associated with an object if it is a large object with more than one hdr
   blk */
#ifdef OLD
#define SetAllocated(ptr)   ((ptr)->iRegionSize |= 0x00000001)
#define ClearAllocated(ptr) ((ptr)->iRegionSize &= 0xfffffffe)
#endif

#define SetAllocated(ptr) ((ptr)->iRegionSize |= 0x00000001); \
if ((ptr)->iRegionSize > BLOCKSIZE) \
{ int iLastHdrBlk = (int) GET_LAST_HDR_BLK((ptr), RegionSize(ptr)); \
	((BlockStructPtr)(iLastHdrBlk))->iRegionSize = \
	  (int)((BlockStructPtr)(iLastHdrBlk))->iRegionSize | 1; \
}


#define ClearAllocated(ptr) (((ptr)->iRegionSize) &= ~1); \
if ((ptr)->iRegionSize > BLOCKSIZE) \
{ int iLastHdrBlk = (int) GET_LAST_HDR_BLK((ptr), RegionSize(ptr)); \
	((BlockStructPtr)(iLastHdrBlk))->iRegionSize = \
	  (int)((BlockStructPtr)(iLastHdrBlk))->iRegionSize & ~1; \
}


#define GetInPartiallyFilledBlockBin(ptr) ((ptr)->iRegionSize &  0x00000002)
#define SetInPartiallyFilledBlockBin(ptr) ((ptr)->iRegionSize |= 0x00000002)
#define ClearInPartiallyFilledBlockBin(ptr) ((ptr)->iRegionSize &= 0xfffffffd)


/**************************************************************
 Multiple large objects of the same size will be hung off a node in the
   binary tree.  The start of this list will be pointed to by the pLLHeadOrNext
   pointer.  Elements in the link list will use this field as the next pointer.
   The strategy is as follows:
            Binary Tree Node                   Link List Node
            =====================              =========================
pLLHeadOrNext  Start of Link List or NULL       Next element in link list
pRightChild    Right Child                      Previous (parent)
pLeftChild     Left Child (or NULL)             ~0x0

***************************************************************/

#define LL_FLAG ((BlockStructPtr) ~0x0)
#define IS_REGULAR_NODE(ptr) (LLHeadOrNext(ptr) == (BlockStructPtr) NULL && \
                              LeftChild(ptr) != LL_FLAG)

#define IS_LL_NODE(ptr) (LeftChild(ptr) == LL_FLAG)

#define IS_LL_ROOT(ptr) (LLHeadOrNext(ptr) != (BlockStructPtr) NULL && \
                              LeftChild(ptr) != LL_FLAG)

/* Calculate the address of a chunk in a block of memory */
#define ChunkInBlock(pBlock, index, iSize) ((long *)((char *)(pBlock) + index*iSize))
/* Get the last chunk in a block.  The index is 1 less than the number of
   chunks which will fit into a block.  pFreeBlock is a header block. */
#define GetLastChunk(pFreeBlock) (ChunkInBlock(HEADER_TO_MEMORY(pFreeBlock), \
							 BLOCKSIZE/ChunkSize(pFreeBlock) - 1, \
							 ChunkSize(pFreeBlock)))

#define GETPARTIALBIN(iSize) (apPartiallyFilledBlockVector[(int) GetActualIndex(iSize)])

/* Routines to lock and unlock mutex associated with a bin for actual object */
/* size */
	
#define LOCK_ACTUALBIN_MUTEX(iSize) \
 (sysMonitorEnter(maActualBinMutex[(int) GetActualIndex(iSize)]))
#define UNLOCK_ACTUALBIN_MUTEX(iSize) \
  (sysMonitorExit(maActualBinMutex[(int) GetActualIndex(iSize)]))

#define LOCK_BIN(index,ee)   sysMonitorEnterQuicker(maActualBinMutex[index],SYSTHREAD(ee->thread))
#define UNLOCK_BIN(index,ee) sysMonitorExitQuicker (maActualBinMutex[index],SYSTHREAD(ee->thread))

  
	 /* TMR - mutex check*/
	 /*
#define LOCK_ACTUALBIN_MUTEX(iSize) \
 (pthread_mutex_lock(&(maActualBinMutex[(int) GetActualIndex(iSize)])) ? \
  errno : 0)

#define UNLOCK_ACTUALBIN_MUTEX(iSize) \
 (pthread_mutex_unlock(&(maActualBinMutex[(int) GetActualIndex(iSize)])) \
  ? errno :0 )
  */

     /* This may need to be updated if the bitmap or BLOCKSIZE changes */   /* Eitan */ 
#define BITMAP_BYTES_PER_BLOCK 256  
/* This is used for unrolling and scrambling in HMchunkBlock */    /* Eitan */
#define LINK_GAP 4                  
       
     /* #define BLOCKSIZE ((int) (getpagesize()))  */
#define BLOCKSIZE 4096
#define MAX_SMALL_OBJECT ((int)(BLOCKSIZE/2))

#define MAX_ALLOCATION_TRIES 3  /* Number of times to attempt synch GC */

#define NORMALIZE_HEAP(pHeapAddr) ((char *)(pHeapAddr) - (char *)heapbase)
#define NORMALIZE_HDR_BLK(pHdrBlkAddr) \
((char *)(pHdrBlkAddr) - (char *) vpHeaderBlkStart)
#define HEADER_TO_MEMORY1(pHeader) \
(void *)(((NORMALIZE_HDR_BLK(pHeader)/sizeof(blockStruct))*BLOCKSIZE) + (char *)heapbase)

#define HEADER_TO_MEMORY(pHeader) \
((void *)(((NORMALIZE_HDR_BLK(pHeader)>>4)<<12) + (char *) heapbase))

#define MEMORY_TO_HEADER1(pMem) \
((BlockStructPtr)((((char *) (pMem) - heapbase)/BLOCKSIZE)*sizeof(blockStruct) + (char *) vpHeaderBlkStart))

#define MEMORY_TO_HEADER(pMem) \
((BlockStructPtr)((((NORMALIZE_HEAP(pMem))>>12)<<4) + (char *) vpHeaderBlkStart))

//int nbrSweeps;


/* Change the definition here for MVS port */
/*
#define COMPARE_AND_SWAP(word_addr, old_val_addr, new_val) \
      (!_check_lock((atomic_p) word_addr, (int) old_val_addr, (int) new_val))
	  */
//#define ASSERT(assertion) if (assertion) ; else assert(assertion)
#define ASSERT(assertion) sysAssert(assertion)

/* Replace this with the JVM mechanism */
#define FATAL_ERROR(string) fprintf(stderr, "string\n"); \
 exit(-1)

typedef enum Childtype {SAME, LEFTCHILD, RIGHTCHILD} childtype;
#ifndef False   /*ibm.7179*/
typedef enum Boolean {False=0, True} boolean;
#endif   /*ibm.7179*/
typedef struct LargeObject *LargeObjPtr;
typedef struct SmallObject *SmallObjPtr;


/* Note that the allocated flag is stored in the LSB bit of iRegionSize.
   The Unused pointer is added to expand the size up to 16 bytes so we can use a
   shift operation instead of division. */

typedef struct blockStruct *BlockStructPtr;

typedef struct LargeObject {
  BlockStructPtr pLeftChild;
  BlockStructPtr pRightChild;
  BlockStructPtr pLLHeadOrNext;   /* Either ptr to ll head, or next node */
} LargeObject;

typedef struct SmallObject {
  BlockStructPtr pPrevBlock;
  BlockStructPtr pNextBlock;
  char *Unused;
} SmallObject;  

typedef struct blockStruct {
  int iRegionSize;
  union {
	SmallObject udSmallObj;
	LargeObject udLargeObj;
  } u;
} blockStruct;

/* ibm.5833 TMR - 6/98
 * -------------------Sticky Bits macros --------------------------------
 * The Dirty Bit is the last (LSB) in the third pointer in the block header.
 * In large object the third pointer is in use only when the block is not
 * allocated.
 * In small object this pointer is not in use at all*/
#define markDirtyBit(mem) (MEMORY_TO_HEADER(mem))->u.udLargeObj.pLLHeadOrNext = (BlockStructPtr)1;

#define markDirtyBit_ee(mem,ee)  \
{ \
((char*)ee->compensatedBlockHeaders)[(((unsigned int)mem>>12)<<4)+12]=1; \
}

#define clearDirtyBit(ud) (ud)->u.udLargeObj.pLLHeadOrNext = 0;
    
#define isBitDirty(ud) (ud)->u.udLargeObj.pLLHeadOrNext

/* ITL - (2/98) */
/* ----------------- Common gc routines & variables ---------------------- */
#ifdef ALLOC_BARRIERS
void DoASChangeCallback(int as_old, int as_new);
extern volatile int allocstate;
extern long _barriersize_red;
extern long _barriersize_yellow;
#define ALLOCSTATE_GREEN   1
#define ALLOCSTATE_YELLOW  2
#define ALLOCSTATE_RED     3
#endif /* ALLOC_BARRIERS */


/* ----------------------- Heap Manager global variables ---------------- */
extern BlockStructPtr pTreeStart;  /* Start of binary tree */
//extern void *pWildernessStart;  /* Start of wilderness area */
extern int iFreeBytes;   /* Number of free bytes left in our Java heap */
extern int iLastAsync;   /* Cycle number of sweep when last GC occurred */
extern int iNumSweeps;   /* Number of sweeps executed */
extern void *vpHeaderBlkStart;  /* start of header area in allocated heap */
//extern void *vpColorMapStart; /* Start of color map */
extern void *heapbase;        /* Start of Java heap */

/* ------------------------ Binary tree routines ------------------------ */

BlockStructPtr HMremoveMin(BlockStructPtr pStartSearch);
BlockStructPtr HMfindAndRemove(int iRequestedNumBlocks, BlockStructPtr p);
int HMremoveNode(BlockStructPtr pRemoveMe, BlockStructPtr *p);
int HMinsert(BlockStructPtr udNewElement);
/*ibm.6565*/
extern void * HMallocate(int iSize,
                         int index,
                         struct methodtable *mptr,
                         int obj_type,
                         ExecEnv *ee);  
/*ibm.6565*/
/* --------------------- Common Routines --------------------------------*/
extern void * HMallocateBlocks(int iNumBlocks);
extern BlockStructPtr HMremoveFromWilderness(int iNumBlocks);
extern void HMfreeBlocks(BlockStructPtr p);
extern BlockStructPtr HMremoveSize(int iSize);
extern BlockStructPtr HMremoveAddress(BlockStructPtr pHdrBlk);
/* ITL (2/98) */
#ifdef ALLOC_BARRIERS
extern long HMlargestContiguousChunk();
extern long HMlargestElementInTree();
extern int HMcurrentAllocationState();
extern int HMtestIfAllocationStateChanged();
#endif /* ALLOC_BARRIERS */

#ifdef PURPLE_ASYNC_TRIGGER
extern int iBlocksInPartially[sizeof(binsizes)/sizeof(int)]; 
extern int iLargestBin;
extern int iBlocksInLargest;
#define MIN_BLOCK_IN_BIN (int)(iBlocksInLargest*0.2)
#define MIN_BLOCK_AFTER_SWEEP 50
#define SMALLEST_BIN_CHECKED GetActualIndex(128)
#endif 

void HMstartSweep();                                 /*ibm.5404*/


/*ibm.6565*/
#define INDEX 0
#define ACTUALSIZE 1

#define GetActualIndex(requestedSize) aGeneralBinVector[requestedSize][INDEX]
#define AllocateSize(size) aGeneralBinVector[size][ACTUALSIZE]
#define SizeToAllocationSize(n0) (n0 + HANDLE_SIZE + (OBJECTGRAIN - 1)) & ~(OBJECTGRAIN - 1)

extern int aGeneralBinVector[MAX_SMALL_OBJECT+1][2];

#define InitializeChunk(__vpChunk,__mptr,__obj_type,__ee)       \
{                                                               \
  __vpChunk->methods = __mptr;                                  \
  __vpChunk->locknflags = obj_makelocknflags(__obj_type,        \
                            __obj_type ? OBJ_IsArray : 0);      \
  __ee->cantCooperate=TRUE;                                     \
  SET_COLOR(__vpChunk,allocationColor);                         \
  __ee->cantCooperate=FALSE;                                    \
  sysAssert(ValidHandle(__vpChunk));                            \
}

#define TOTAL_BIN_SIZES 32

extern int binsizes[TOTAL_BIN_SIZES];

#ifdef IBM_MVS
/* ibm.7978 Kathy -- dynamic bin vectors number, start */
extern void * volatile *paActualBinVectorArray; 

#define EESetBin(__ee,__n)                                  \
{                                                           \
  __ee->AllocVecNum = __n ;                                 \
  __ee->pBin = (JHandle (* volatile (*) ) )                 \
                &paActualBinVectorArray[(__ee->AllocVecNum)*TOTAL_BIN_SIZES];  \
}
 
/* ibm.7978 Kathy -- dynamic bin vectors number, end */
#endif /* IBM_MVS */

/*
 * Use __ for all identifiers to avoid aliasing.
 */ 

#define allocate(__mptr,                                                        \
                 __iSize,                                                       \
                 __index,                                                       \
                 __obj_type,                                                    \
                 __ee,                                                          \
                 __vpChunk)                                                     \
{                                                                               \
    if( __index >= 0 )                                                          \
      {                                                                         \
        JHandle           *__vpNextChunk;                                       \
        JHandle *volatile *__pBin=&__ee->pBin[__index];                         \
                                                                                \
        __vpChunk = *__pBin;                                                    \
        if(!__vpChunk)                                                          \
          goto call;                                                            \
        sysAssert(MaybeValidHandle(__vpChunk));                                 \
                                                                                \
        __vpNextChunk = (JHandle *)NextChunk(__vpChunk);                        \
        sysAssert((!__vpChunk)||MaybeValidHandle(__vpChunk));                   \    
                                                                                \
        if (!CMP_AND_SWP(__pBin, __vpChunk, __vpNextChunk))                     \
          goto call;                                                            \
                                                                                \
        sysAssert(*__pBin!=__vpChunk);                                          \
                                                                                \
        InitializeChunk(__vpChunk,__mptr,__obj_type,__ee);                      \
        goto exit;                                                              \
                                                                                \
      }                                                                         \
 call:                                                                          \
    __vpChunk=(JHandle*)(HMallocate(__iSize,__index,__mptr,__obj_type,__ee));   \
 exit:                                                                          \
    ;                                                                           \
}

/*ibm.6565*/

#endif
