/*
 * @(#)atoe.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/*
 * @(#)atoe.h	1.00 97/03/04
 * 
 * @STANDARD_IBM_COPYRIGHT@
 *
 * DESCRIPTION:
 * Undefine and redefine all IO functions to install ascii <-> ebcdic
 * conversion.
 *
 * NOTE: The following fuctions are redefined in platform specific code.
 * gethostbyname_r, gethostbyaddr_r, gethostname, dllload.
 *
 * The following functions are NOT redefined :
 * getchar, asctime, ctime, strtol, strtoul, atof, atoi, atol
 * 
 * CHANGES:
 * 04Mar1997 nnnnn PHA  Separated generic code from platform specific
 */

#ifndef _ATOE_H_
#define _ATOE_H_

#include <dirent.h>
#include <stdio.h>
#include "atoe_platform.h"

DIR*           atoe_opendir   (const char*);
FILE*          atoe_fopen     (const char*, char*);
char*          atoe_getcwd    (char*, size_t);
char*          atoe_getenv    (const char*);
int            atoe_access    (const char*, int);
int            atoe_fprintf   (FILE*, const char*, ...);
int            atoe_mkdir     (const char*, mode_t);
size_t         atoe_fread     (void*, size_t, size_t, FILE*);
size_t         atoe_fwrite    (void*, size_t, size_t, FILE*);
struct passwd* atoe_getpwuid  (uid_t);
void           atoe_perror    (const char*);
int            atoe_open      (const char*, int, ...);
int            atoe_printf    (const char*, ...);
int            atoe_putchar   (int);
int            atoe_putenv    (const char*);
int            atoe_rename    (const char*, char*);
int            atoe_sprintf   (const char*, char*, ...);
int            atoe_stat      (const char*, struct stat*);
char*          atoe_strerror  (int);
size_t         atoe_strftime  (char*, size_t,const char*, const struct tm*);
int            atoe_unlink    (const char*);
void           atoe_vfprintf  (FILE *, const char *, va_list);
char*          atoe_tempnam   (const char *, const char *);
double         atoe_strtod    (char *,char **);
int            atoe_strcasecmp(const char *string1, const char *string2);


#undef access
#undef fopen
#undef fprintf
#undef fread
#undef fwrite
#undef getcwd
#undef getenv
#undef getpwuid
#undef mkdir
#undef open
#undef opendir
#undef perror
#undef printf
#undef putchar
#undef putenv
#undef rename
#undef sprintf
#undef stat
#undef strcasecmp
#undef strerror
#undef strftime
#undef strtod
#undef tempnam
#undef unlink
#undef vfprintf

#define access(a,b)     atoe_access(a,b)
#define fopen           atoe_fopen
#define fprintf         atoe_fprintf
#define fread           atoe_fread
#define fwrite          atoe_fwrite
#define getcwd          atoe_getcwd
#define getenv          atoe_getenv
#define getpwuid        atoe_getpwuid
#define mkdir           atoe_mkdir
#define open            atoe_open
#define opendir         atoe_opendir
#define perror          atoe_perror
#define printf          atoe_printf
#define putchar         atoe_putchar
#define putenv          atoe_putenv
#define rename          atoe_rename
#define rslvsp          atoe_rslvsp
#define sprintf         atoe_sprintf
#define stat(a,b)       atoe_stat(a,b)
#define strcasecmp      atoe_strcasecmp
#define strerror        atoe_strerror
#define strftime        atoe_strftime
#define strtod          atoe_strtod
#define strtod          atoe_strtod
#define system          atoe_system
#define tempnam         atoe_tempnam
#define unlink          atoe_unlink
#define vfprintf        atoe_vfprintf

#endif	/* _ATOE_H_ */
