/*
 * @(#)exnonex_md.h     97/02/19
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
** EXCLUSIVE/NON-EXCLUSIVE locking by Stephen Blackheath, 19 Feb 97
** (c) Copyright IBM Corporation, 1996-1997. All rights reserved.
**
** A non-exclusive lock gives code non-exclusive access to some resource.
** More than one thread can have the same non-exclusive lock at the same time,
** so if ONLY non-exclusive locks are used, it is equivalent to using no lock
** at all.  The power comes in with the exclusive lock.  
**
** An exclusive lock grants exclusive access to the resource, thus excluding
** any areas of code which take non-exclusive locks on the same 'exnonex'.
**
** This is especially powerful where you have several threads which read
** something, and, since they don't interfere with each other, can happily
** run at the same time as each other.  When you want to change the same
** resource, you can lock out all the reading threads by taking an exclusive
** lock on the resource.
**
** The ultimate reason for creating this locking system is speed.  A
** non-exclusive lock can be grabbed in approximately 200 ns.
** The exclusive lock is slower than a normal pthread_mutex, so this system
** is only useful where most locks are non-exclusive.
*/

#ifndef POSIX_EXNONEX_MD_H
#define POSIX_EXNONEX_MD_H

#include	<pthread.h>

typedef struct sys_exnonex {
  int state;
  pthread_mutex_t lock;
  pthread_mutex_t wait;
  pthread_cond_t cond;
} sys_exnonex_t;

  /* Initialize the lock */
void sysExNonexLockInit(sys_exnonex_t* mon);

  /* Lock and unlock exclusively */
void sysExclusiveLock(sys_exnonex_t* mon);
void sysExclusiveUnlock(sys_exnonex_t* mon);

  /* Lock and unlock non-exclusively */
void sysNonexclusiveLock(sys_exnonex_t* mon);
void sysNonexclusiveUnlock(sys_exnonex_t* mon);

#endif
