/*
 * @(#)float_md.h       1.00 97/02/25
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _UNIX_FLOAT_H_
#define _UNIX_FLOAT_H_

#include <longlong_md.h>

#ifndef IBM_LINUX
#undef  float_t
#undef  double_t
#endif

#if defined(IBM_AIX) || defined(IBM_LINUX)               /* ibm.3205 */
#define float2l(f) (flt2int(f))         /* ibm.3205 */
#define double2l(d) (dbl2int(d))        /* ibm.3205 */
#define MIN_INT                         0x80000000	/* ibm.12687 */
#define MAX_INT                         0x7FFFFFFF	/* ibm.12687 */

#endif                                  /* ibm.3205 */

/*
 * NAN_CHECK() operands l and r are both float_t or both double_t
 * while operand x is a multiple ternary expression involving l and r.
 * The expression must be insensitive to whether IEEE 754 support is
 * native or emulated.
 */
#define NAN_CHECK(l,r,x) x

#ifdef  NEEDSIEEE754
        #define NEEDSPACKSTRUCT                                 /*ibmu02602*/
        /*
         *
         */
        #include <packon_md.h>                                  /*ibmu02602*/
        MVSPACK struct  ieee_sp         {
                unsigned int    sign:1;
                unsigned int    expo:8;
                unsigned int    frac:23;
                };
        #include <packoff_md.h>                                 /*ibmu02602*/

        /*
         *
         */
        #include <packon_md.h>                                  /*ibmu02602*/
        MVSPACK struct  ieee_dp         {
                unsigned int    sign:1;
                unsigned int    expo:11;
                unsigned int    frah:20;
                unsigned int    fral:32;
                };
        #include <packoff_md.h>                                 /*ibmu02602*/

        /*
         *
         */
        #include <packon_md.h>                                  /*ibmu02602*/
        MVSPACK struct  lbitf {
                unsigned int    sign:1;
                unsigned int    expo:8;
                unsigned int    frac:22;
                unsigned int    low:1;
                        };
        #include <packoff_md.h>                                 /*ibmu02602*/

        /*
         *
         */
        #include <packon_md.h>                                  /*ibmu02602*/
        MVSPACK struct  lbitd {
                unsigned int    sign:1;
                unsigned int    expo:11;
                unsigned int    frah:20;
                unsigned int    high:1;
                unsigned int    frac:30;
                unsigned int    low:1;
                        };
        #include <packoff_md.h>                                 /*ibmu02602*/

        /*
         * This structure is used to allow loading emulated doubles
         * from 2 32bit int's see below DBL_DEF & DBL_VAL
         */
        #include <packon_md.h>                          /*ibm.4763*/
        MVSPACK struct  load_32bit {                    /*ibm.4763*/
                unsigned int    h:32;                   /*ibm.4763*/
                unsigned int    l:32;                   /*ibm.4763*/
                        };                              /*ibm.4763*/
        #include <packoff_md.h>                         /*ibm.4763*/

        /*
         *
         */
        typedef union   float_t {
                        long    i;
                        unsigned long   u;
                        float   v;          /* debug use only */
                        struct ieee_sp  m;
                        struct lbitf    l;
                } float_t;

        typedef union   double_t {
                        struct load_32bit x;            /*ibm.4763*/
                        int64_t i;
                        double  v;          /* debug use only */
                        struct ieee_dp m;
                        struct lbitd    l;
                } double_t;

        /*
         *
         */
        #define DBL_DEF(a,b,c,d)        double_t a = { (c) , (d) }
        #define DBL_VAL(a,b,c  )        { (b) , (c) }
        #define FLT_DEF(a,b,c  )        float_t  a = { (c) }

        /*
         *
         */
        double_t        int2dbl(int a);
        float_t         int2flt(int a);

        double_t        ll2double(int64_t a);
        float_t         ll2float(int64_t a);

        int32_t         dbl2int(double_t d);                    /*ibm01358*/
        int64_t         double2ll(double_t d);
        float_t         dbl2flt(double_t d);

        int32_t         flt2int(float_t d);                     /*ibm01358*/
        int64_t         float2ll(float_t d);
        double_t        flt2dbl(float_t f);

        float_t         str2flt(char *buf, char **dmy);
        double_t        str2dbl(char *buf, char **dmy);

        char *          flt2str(char *buf, const char * patt, float_t f);
        char *          dbl2str(char *buf, const char * patt, double_t d);

        /*
         *
         */
        double_t        dbl_neg(double_t d);
        double_t        dbl_copysign(double_t a, double_t b);
        double_t        dbl_add(double_t a, double_t b);
        double_t        dbl_sub(double_t a, double_t b);
        double_t        dbl_mul(double_t a, double_t b);
        double_t        dbl_div(double_t a, double_t b);

        int             dbl_eqz(double_t a);
        int             dbl_ltz(double_t a);
        int             dbl_gez(double_t a);
        int             dbl_eq(double_t a, double_t b);
        int             dbl_ne(double_t a, double_t b);
        int             dbl_ge(double_t a, double_t b);
        int             dbl_le(double_t a, double_t b);
        int             dbl_lt(double_t a, double_t b);
        int             dbl_gt(double_t a, double_t b);

        float_t         flt_neg(float_t d);
        float_t         flt_add(float_t a, float_t b);
        float_t         flt_sub(float_t a, float_t b);
        float_t         flt_mul(float_t a, float_t b);
        float_t         flt_div(float_t a, float_t b);

        int             flt_eqz(float_t a);
        int             flt_ltz(float_t a);
        int             flt_gez(float_t a);
        int             flt_eq(float_t a, float_t b);
        int             flt_ne(float_t a, float_t b);
        int             flt_ge(float_t a, float_t b);
        int             flt_le(float_t a, float_t b);
        int             flt_lt(float_t a, float_t b);
        int             flt_gt(float_t a, float_t b);

        /*
         *
         */
        float_t         tofloat(float a);
        double_t        todouble(double a);
        float           flt2nat(float_t a);
        double          dbl2nat(double_t a);

        /*
         *
         */
        double_t        dbl_cos(double_t a);
        double_t        dbl_sin(double_t a);
        double_t        dbl_tan(double_t a);
        double_t        dbl_asin(double_t a);
        double_t        dbl_acos(double_t a);
        double_t        dbl_atan(double_t a);
        double_t        dbl_exp(double_t a);
        double_t        dbl_log(double_t a);
        double_t        dbl_sqrt(double_t a);
        double_t        dbl_ceil(double_t a);
        double_t        dbl_floor(double_t a);
        double_t        dbl_rint(double_t a);
        double_t        dbl_atan2(double_t a, double_t b);
        double_t        dbl_pow(double_t a, double_t b);
        double_t        dbl_fmod(double_t a, double_t b);
        double_t        dbl_rem(double_t a, double_t b);

        float_t         flt_fmod(float_t a, float_t b);

        /*
         *
         */
        #define dbl_zero_const  PosZerod
        #define dbl_one_const   PosOned
        #define dbl_max_const   PosMaxd
        #define dbl_min_const   PosMind

        #define flt_zero_const  PosZerof
        #define flt_one_const   PosOnef

        extern  double_t        PosZerod;
        extern  double_t        PosOned;
        extern  double_t        PosInfd;
        extern  double_t        PosMaxd;
        extern  double_t        PosMind;
        extern  float_t         PosZerof;
        extern  float_t         PosOnef;
        extern  float_t         PosInff;
        extern  float_t         PosMaxf;
        extern  float_t         PosMinf;

        extern  double_t        NegZerod;
        extern  double_t        NegOned;
        extern  double_t        NegInfd;
        extern  float_t         NegZerof;
        extern  float_t         NegOnef;
        extern  float_t         NegInff;

        #define SINGLE_BIAS                     127
        #define DOUBLE_BIAS                     1023
        #define SINGLE_SIZE                     24
        #define DOUBLE_SIZE                     53

        #define EXPONENT_IS_ZERO(x)             ((x).m.expo == 0)
        #define POS(x)                          ((x).m.sign == 0)
        #define SET_SIGN(x,y)                   ((x).m.sign = (y))
        #define GET_SIGN(x)                     ((x).m.sign)
        #define SET_HIDDEN_BIT(x)               ((x).m.expo = ((x).m.expo != 0))

        #define DOUBLE_EXPONENT_IS_ONES(x)      ((x).m.expo  == ((DOUBLE_BIAS*2)+1))
        #define DOUBLE_FRACTION_IS_ZERO(x)      (((x).m.fral == 0) && ((x).m.frah == 0))
        #define DOUBLE_FRACTION(x)              ((x).i)
        #define DOUBLE_TO_SINGLE_FRAC(x)        (ll2int(ll_ushr(DOUBLE_FRACTION((x)),DOUBLE_SIZE-SINGLE_SIZE)))
        #define GET_DOUBLE_EXPONENT(x)          ((x).m.expo-DOUBLE_BIAS)
        #define SET_DOUBLE_EXPONENT(x,y)        ((x).m.expo = (y)+DOUBLE_BIAS)
        #define DOUBLE_IS_NAN(x)                (DOUBLE_EXPONENT_IS_ONES(x) && !DOUBLE_FRACTION_IS_ZERO(x))
        #define DOUBLE_IS_INFINITE(x)           (DOUBLE_EXPONENT_IS_ONES(x) &&  DOUBLE_FRACTION_IS_ZERO(x))
        #define DOUBLE_IS_ZERO(x)               (EXPONENT_IS_ZERO(x) && DOUBLE_FRACTION_IS_ZERO(x))

        #define SINGLE_EXPONENT_IS_ONES(x)      ((x).m.expo == ((SINGLE_BIAS*2)+1))
        #define SINGLE_FRACTION_IS_ZERO(x)      ((x).m.frac == 0)
        #define SINGLE_FRACTION(x)              ((x).i)
        #define SINGLE_TO_DOUBLE_FRAC(x)        (ll_shl(uint2ll(SINGLE_FRACTION(x)),DOUBLE_SIZE-SINGLE_SIZE))
        #define GET_SINGLE_EXPONENT(x)          ((x).m.expo-SINGLE_BIAS)
        #define SET_SINGLE_EXPONENT(x,y)        ((x).m.expo = (y)+SINGLE_BIAS)
        #define SINGLE_IS_NAN(x)                (SINGLE_EXPONENT_IS_ONES(x) && !SINGLE_FRACTION_IS_ZERO(x))
        #define SINGLE_IS_INFINITE(x)           (SINGLE_EXPONENT_IS_ONES(x) &&  SINGLE_FRACTION_IS_ZERO(x))
        #define SINGLE_IS_ZERO(x)               (EXPONENT_IS_ZERO(x) && SINGLE_FRACTION_IS_ZERO(x))

        #define ROUNDHIGH(x)                    (((ll_low(x)&0x80000000)&&((ll_low(x)&0x7fffff00)||(ll_high(x)&0x01))))

        /*ibm01358*/
        #define IS_NAND(x)      DOUBLE_IS_NAN(x)
        #define IS_NANF(x)      SINGLE_IS_NAN(x)

        extern  double_t        NaNd;
        extern  float_t         NaNf;

        /*
         *
         */
        #ifndef MAXFLOAT
        #define MAXFLOAT        dbl_max_const
        #else
        #undef  MAXFLOAT
        #define MAXFLOAT        dbl_max_const
        #endif

        #ifndef MINFLOAT
        #define MINFLOAT        dbl_min_const
        #else
        #undef  MINFLOAT
        #define MINFLOAT        dbl_min_const
        #endif

#else    /* NEEDSIEEE754*/

        /*
         *
         */
#ifndef IBM_LINUX
        typedef float                   float_t;
        typedef double                  double_t;
#endif
        #define DBL_DEF(a,b,c,d)        double_t a = (b)
        #define DBL_VAL(a,b,c  )        (a)
        #define FLT_DEF(a,b,c  )        float_t  a = (b)

        #define int2dbl(a)              ((double_t)(a))
        #define int2flt(a)              ((float_t)(a))

        #define dbl2flt(a)              ((float)(a))

        #define flt2dbl(a)              ((double)(a))

        #define str2flt(a,b)            (float_t)strtod(a,b)
        #define str2dbl(a,b)            strtod(a,b)

        char *  flt2str(char *buf, const char * pattern, float_t f);
        char *  dbl2str(char *buf, const char * pattern, double_t d);

        #define dbl_neg(a)              (-(a))
        #define dbl_copysign(a,b)       (copysign((a),(b)))
        #define dbl_add(a,b)            ((a)+(b))
        #define dbl_sub(a,b)            ((a)-(b))
        #define dbl_mul(a,b)            ((a)*(b))
        #define dbl_div(a,b)            ((a)/(b))

        #define dbl_eqz(a)              ((a)==0.0)
        #define dbl_ltz(a)              ((a)<0.0)
        #define dbl_gez(a)              ((a)>=0.0)
        #define dbl_eq(a,b)             ((a) == (b))
        #define dbl_ne(a,b)             ((a) != (b))
        #define dbl_ge(a,b)             ((a) >= (b))
        #define dbl_le(a,b)             ((a) <= (b))
        #define dbl_lt(a,b)             ((a) <  (b))
        #define dbl_gt(a,b)             ((a) >  (b))

        #define flt_neg(a)              (-(a))
        #define flt_add(a,b)            ((a)+(b))
        #define flt_sub(a,b)            ((a)-(b))
        #define flt_mul(a,b)            ((a)*(b))
        #define flt_div(a,b)            ((a)/(b))

        #define flt_eqz(a)              ((a)==0.0)
        #define flt_ltz(a)              ((a)<0.0)
        #define flt_gez(a)              ((a)>=0.0)
        #define flt_eq(a,b)             ((a) == (b))
        #define flt_ne(a,b)             ((a) != (b))
        #define flt_ge(a,b)             ((a) >= (b))
        #define flt_le(a,b)             ((a) <= (b))
        #define flt_lt(a,b)             ((a) <  (b))
        #define flt_gt(a,b)             ((a) >  (b))

        #define tofloat(a)              ((float_t)(a))
        #define todouble(a)             ((double_t)(a))
        #define flt2nat(a)              ((a))
        #define dbl2nat(a)              ((a))

        #define dbl_cos                 cos
        #define dbl_sin                 sin
        #define dbl_tan                 tan
        #define dbl_asin                asin
        #define dbl_acos                acos
        #define dbl_atan                atan
        #define dbl_exp                 exp
        #define dbl_log                 log
        #define dbl_sqrt                sqrt
        #define dbl_ceil                ceil
        #define dbl_floor               floor
        #define dbl_rint                rint
        #define dbl_atan2               atan2
        #define dbl_pow                 pow
        #define dbl_fmod                fmod
        #define dbl_rem                 remainder

        #define flt_fmod                fmod

        #define dbl_zero_const          ((double_t)0)
        #define dbl_one_const           ((double_t)1)

        #define flt_zero_const          ((float_t)0)
        #define flt_one_const           ((float_t)1)

        #define PosZerod                ((double_t)0)
        #define PosOned                 ((double_t)1)
        #define PosInfd                 ((double_t)INF)
        #define PosZerof                ((float_t)0)
        #define PosOnef                 ((float_t)1)
        #define PosInff                 ((float_t)INF)

        #define NegZerod                -((double_t)0)
        #define NegOned                 -((double_t)1)
        #define NegInfd                 -((double_t)INF)
        #define NegZerof                -((float_t)0)
        #define NegOnef                 -((float_t)1)
        #define NegInff                 -((float_t)INF)

        /*
         * AIX isnan() takes a double argument, no native isnanf().
         */
        #if defined(AIX) || defined(IBM_IEEE_HSP) || defined(IBM_LINUX)       /*ibm.5429*/
                #define IS_NAND(x)      isnan(x)
                #define IS_NANF(x)      isnan(x)
        #else
                #define IS_NAND(x)      isnand(x)
                #define IS_NANF(x)      isnanf(x)
        #endif

        #define NaNd                    ((double_t)NAN)
        #define NaNf                    ((float_t)NAN)

#endif    /*NEEDSIEEE754*/
#endif
