/*
 * @(#)jni_md.h 1.2 96/10/24
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#if defined(linux) && !defined(IBM_LINUX) /* ibm.6843 */
#define MON_FLAT /* ibm.6843 */
#include "oobj_md.h" /* ibm.6843 */
#define IBM_LINUX /* ibm.6843 */
#endif /* ibm.6843 */


#include "float_md.h"

#define JNIEXPORT
#define JNICALL

typedef signed char jbyte;
typedef long jint;
typedef int64_t jlong;                                             /*ibm.7280*/

#ifndef IBM_LINUX 

/* ibm.8774.8545:

   jniFlag is changed both here in the jni, and when suspending/resuming where
   the suspend interest flag is affected. The ENTERING_JVM and LEAVING_JVM
   macros block until the suspend/resume bit is zero and then if ENTERING_JVM,
   set the jni bit of jniFlag to 0 to make the thread suspendable, and if
   LEAVING_JVM, set the jni bit of jniFlag to 1 to make the thread 
   unsuspendable. This ensures that changing the jni bit is synchronous
   with checking the jni bit before deciding to suspend or resume. 

   The thread stackptr is saved when the thread is about to leave the jvm and 
   become unsuspendable. There is no synchronization here because I don't
   think any is needed. If the gc is invoked before or during an update
   to stackptr, then the thread will suspend and _doSuspendLoop will update
   stackptr. The stackptr will return to its pre-interrupted value after
   _doSuspendLoop and the update here will be unaffected.

   A third flag has been added to jniFlag to handle nested calls to jni_
   functions, and jni_ functions called from within the jvm. This flag
   (the Ignore flag) if set means the sigsetjmp and compare_and_swap
   loops are not executed. In other words there are cases where a jni_
   function is called where the jvm is not really being entered or left and
   this is where the ignore flag is used. 
*/
/* ibm.8774.8545, ibm.8687 - start */
#define IBM_IGNORE_FLAG 0x1fffffff
#define IBM_SET_IGNORE_FROM_JNI IBM_SET_IGNORE((sysThreadSelf())->jniFlag)
#define IBM_UNSET_IGNORE_FROM_JNI IBM_UNSET_IGNORE((sysThreadSelf())->jniFlag)
#define IBM_IGNORE(jniFlag) (jniFlag & IBM_IGNORE_FLAG)

#ifdef DEBUG
#define IBM_SET_IGNORE(jniFlag) \
	sysAssert((jniFlag&IBM_IGNORE_FLAG)<IBM_IGNORE_FLAG); \
    fetch_and_add(&(jniFlag), 1)

#define IBM_UNSET_IGNORE(jniFlag) \
    sysAssert((jniFlag&IBM_IGNORE_FLAG)>0); \
    fetch_and_add(&(jniFlag), -1)

#else /* DEBUG */

#define IBM_SET_IGNORE(jniFlag) fetch_and_add(&(jniFlag), 1)
#define IBM_UNSET_IGNORE(jniFlag) fetch_and_add(&(jniFlag), -1)

#endif /* DEBUG */
#define IBM_ENTERING_JVM {\
    sys_thread_t *_tid = sysThreadSelf();\
    if(!IBM_IGNORE(_tid->jniFlag)) {\
        int _currentSuspendState = IBM_JNI_BIT;\
        while(!compare_and_swap(&(_tid->jniFlag), &_currentSuspendState, \
			        0)) {\
	    YIELD;\
	    _currentSuspendState = IBM_JNI_BIT;\
        }\
    }\
}

#define IBM_LEAVING_JVM {\ 
    sys_thread_t *_tid = sysThreadSelf();\
    if(!IBM_IGNORE(_tid->jniFlag)) {\
        sigjmp_buf _context;\
        int _currentSuspendState = 0;\
        sigsetjmp(_context, 1);\
        _tid->stackptr = &_context;\
        while(!compare_and_swap(&(_tid->jniFlag), &_currentSuspendState, \
			        IBM_JNI_BIT)) {\
	    YIELD;\
            _currentSuspendState = 0;\
        }\
    }\
}

/* ibm.8774.8545, ibm.8687 - end */
#endif /* IBM_LINUX */

#if defined _OOBJ_H_ && !defined _PIN_HANDLE_H_                         /*ibm*/

#ifdef __cplusplus
extern "C" {
#endif

/* Temporary place for these functions - <oobj.h> gives us JHandle* */
#define _PIN_HANDLE_H_                                                  /*ibm*/
#ifdef JMS_TRL                                                          /*ibm*/
/* Note: These functions are subject to change, and may be removed */
JHandle* pin_handle(JHandle *h);
void     unpin_handle(JHandle *h);
#else                                                                   /*ibm*/
#define pin_handle(h)   (h)
#define unpin_handle(h) ((void)0)
#endif                                                                  /*ibm*/

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif                                                                  /*ibm*/
