/*
 * @(#)libm_md.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/* @(#)libm_md.h 1.5 95/01/18 */
/*
 This file is a modified version of Sun's original
 distribution intended to be used only in conjunction
 with the platform independent IEEE 754 floating
 point package. The changes made should not prevent 
 the original package from being compiled.
 Alan M. Webb
 29th December 1996
*/

#ifndef _UNIX_LIBM_H_
#define _UNIX_LIBM_H_

#include <float_md.h>

/*
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunSoft, a Sun Microsystems, Inc. business.
 */

#ifdef __NEWVALID	/* special setup for Sun test regime */
#if defined(i386) || defined(i486) || \
	defined(intel) || defined(x86) || defined(i86pc)
#define __LITTLE_ENDIAN
#endif
#endif

#ifdef __LITTLE_ENDIAN
#define __HI(x) *(1+(int*)&x)
#define __LO(x) *(int*)&x
#define __HIp(x) *(1+(int*)x)
#define __LOp(x) *(int*)x
#else
#define __HI(x) *(int*)&x
#define __LO(x) *(1+(int*)&x)
#define __HIp(x) *(int*)x
#define __LOp(x) *(1+(int*)x)
#endif

#ifdef __STDC__
#define	__P(p)	p
#else
#define	__P(p)	()
#endif

/*
 * ANSI/POSIX
 */

extern int signgam;

#ifndef MAXFLOAT
#define	MAXFLOAT	((float_t)3.40282346638528860e+38)
#endif

enum fdversion {fdlibm_ieee = -1, fdlibm_svid, fdlibm_xopen, fdlibm_posix};

#define _LIB_VERSION_TYPE enum fdversion
#define _LIB_VERSION _fdlib_version  

/* if global variable _LIB_VERSION is not desirable, one may 
 * change the following to be a constant by: 
 *	#define _LIB_VERSION_TYPE const enum version
 * In that case, after one initializes the value _LIB_VERSION (see
 * s_lib_version.c) during compile time, it cannot be modified
 * in the middle of a program
 */ 
extern  _LIB_VERSION_TYPE  _LIB_VERSION;

#define _IEEE_  fdlibm_ieee
#define _SVID_  fdlibm_svid
#define _XOPEN_ fdlibm_xopen
#define _POSIX_ fdlibm_posix

struct exception {
	int type;
	char *name;
	double_t arg1;
	double_t arg2;
	double_t retval;
};

#define	HUGE		MAXFLOAT

/* 
 * set X_TLOSS = pi*2**52, which is possibly defined in <values.h>
 * (one may replace the following line by "#include <values.h>")
 */

#define X_TLOSS		1.41484755040568800000e+16 

#define	DOMAIN		1
#define	SING		2
#define	OVERFLOW	3
#define	UNDERFLOW	4
#define	TLOSS		5
#define	PLOSS		6

/*
 * ANSI/POSIX
 */
extern double_t acos __P((double_t));
extern double_t asin __P((double_t));
extern double_t atan __P((double_t));
extern double_t atan2 __P((double_t, double_t));
extern double_t cos __P((double_t));
extern double_t sin __P((double_t));
extern double_t tan __P((double_t));

extern double_t cosh __P((double_t));
extern double_t sinh __P((double_t));
extern double_t tanh __P((double_t));

extern double_t exp __P((double_t));
extern double_t frexp __P((double_t, int *));
extern double_t ldexp __P((double_t, int));
extern double_t log __P((double_t));
extern double_t log10 __P((double_t));
extern double_t modf __P((double_t, double_t *));

extern double_t pow __P((double_t, double_t));
extern double_t sqrt __P((double_t));

extern double_t ceil __P((double_t));
extern double_t fabs __P((double_t));
extern double_t floor __P((double_t));
extern double_t fmod __P((double_t, double_t));

extern double_t erf __P((double_t));
extern double_t erfc __P((double_t));
extern double_t gamma __P((double_t));
extern double_t hypot __P((double_t, double_t));
extern int isnan __P((double_t));
extern int finite __P((double_t));
extern double_t j0 __P((double_t));
extern double_t j1 __P((double_t));
extern double_t jn __P((int, double_t));
extern double_t lgamma __P((double_t));
extern double_t y0 __P((double_t));
extern double_t y1 __P((double_t));
extern double_t yn __P((int, double_t));

extern double_t acosh __P((double_t));
extern double_t asinh __P((double_t));
extern double_t atanh __P((double_t));
extern double_t cbrt __P((double_t));
extern double_t logb __P((double_t));
extern double_t nextafter __P((double_t, double_t));
extern double_t remainder __P((double_t, double_t));
#ifdef _SCALB_INT
extern double_t scalb __P((double_t, int));
#else
extern double_t scalb __P((double_t, double_t));
#endif

extern int matherr __P((struct exception *));

/*
 * IEEE Test Vector
 */
extern double_t significand __P((double_t));

/*
 * Functions callable from C, intended to support IEEE arithmetic.
 */
extern double_t copysign __P((double_t, double_t));
extern int ilogb __P((double_t));
extern double_t rint __P((double_t));
extern double_t scalbn __P((double_t, int));

/*
 * BSD math library entry points
 */
extern double_t expm1 __P((double_t));
extern double_t log1p __P((double_t));

/*
 * Reentrant version of gamma & lgamma; passes signgam back by reference
 * as the second argument; user must allocate space for signgam.
 */
#ifdef _REENTRANT
extern double_t gamma_r __P((double_t, int *));
extern double_t lgamma_r __P((double_t, int *));
#endif	/* _REENTRANT */

/* ieee style elementary functions */
extern double_t __ieee754_sqrt __P((double_t));			
extern double_t __ieee754_acos __P((double_t));			
extern double_t __ieee754_acosh __P((double_t));			
extern double_t __ieee754_log __P((double_t));			
extern double_t __ieee754_atanh __P((double_t));			
extern double_t __ieee754_asin __P((double_t));			
extern double_t __ieee754_atan2 __P((double_t,double_t));			
extern double_t __ieee754_exp __P((double_t));
extern double_t __ieee754_cosh __P((double_t));
extern double_t __ieee754_fmod __P((double_t,double_t));
extern double_t __ieee754_pow __P((double_t,double_t));
extern double_t __ieee754_lgamma_r __P((double_t,int *));
extern double_t __ieee754_gamma_r __P((double_t,int *));
extern double_t __ieee754_lgamma __P((double_t));
extern double_t __ieee754_gamma __P((double_t));
extern double_t __ieee754_log10 __P((double_t));
extern double_t __ieee754_sinh __P((double_t));
extern double_t __ieee754_hypot __P((double_t,double_t));
extern double_t __ieee754_j0 __P((double_t));
extern double_t __ieee754_j1 __P((double_t));
extern double_t __ieee754_y0 __P((double_t));
extern double_t __ieee754_y1 __P((double_t));
extern double_t __ieee754_jn __P((int,double_t));
extern double_t __ieee754_yn __P((int,double_t));
extern double_t __ieee754_remainder __P((double_t,double_t));
extern int    __ieee754_rem_pio2 __P((double_t,double_t*));
#ifdef _SCALB_INT
extern double_t __ieee754_scalb __P((double_t,int));
#else
extern double_t __ieee754_scalb __P((double_t,double_t));
#endif

/* fdlibm kernel function */
extern double_t __kernel_standard __P((double_t,double_t,int));	
extern double_t __kernel_sin __P((double_t,double_t,int));
extern double_t __kernel_cos __P((double_t,double_t));
extern double_t __kernel_tan __P((double_t,double_t,int));
extern int    __kernel_rem_pio2 __P((double_t*,double_t*,int,int,int,const int*));

#endif	/* _UNIX_LIBM_H_ */
