/*
 * @(#)libthread.h	1.6 98/07/01
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 *	struct thread and struct lwp definitions.
 */
#ifndef _LIBTHREAD_
#define	_LIBTHREAD_


/*
 * default stack cache definition.
 */
typedef struct _stkcache {
	int size;
	char *next;
	int busy;
	cond_t cv;
} stkcache_t;


/*
 * used to implement a callout mechanism.
 */
typedef struct callo {
	thread_t tid;
	char flag;
	char running;
	cond_t waiting;
	struct timeval time;
	void (*func)();
	int arg;
	struct callo *forw;
	struct callo *backw;
} callo_t;

typedef	char	thstate_t;

typedef struct thread {
	struct thread	*t_link;	/* run/sleep queue */
	char		*t_stk;		/* stack base */
	unsigned int	t_stksize;	/* size of stack */
	char		*t_tls;		/* pointer to thread local storage */
	long		t_startpc;	/* start func called by thr_create() */
	long		t_pc;		/* resume pc */
	long		t_sp;		/* resume sp */
	thread_t	t_tid;		/* thread id */
	lwpid_t		t_lwpid;	/* lwp id */
	short		t_usropts;	/* usr options, (THR_BOUND, ...) */
	short		t_flag;		/* flags, (T_ALLOCSTK, T_PARK) */
	int		t_pri;		/* scheduling priority */
	/* Keep following 8 fields together. See _clean_thread() */
	thstate_t	t_state;	/* thread state */
	char		t_nosig;	/* block signal handlers */
	char		t_stop;		/* stop thread when set */
	char		t_preempt;	/* preempt thread when set */
	char		t_schedlocked;	/* flag set, thread holding schedlock */
	char		t_pad;		/* dummy pad field */
	char		t_pending;	/* set when t_psig is not empty */
	char		t_bpending;	/* set if signal bouncing needed */
	sigset_t	t_hold;		/* per thread signal mask */
	sigset_t	t_lwpmask;	/* current lwp's signal mask (mirror) */
	sigset_t	t_psig;		/* pending signals */
	sigset_t	t_ssig;		/* signals sent, still pending */
	char		*t_wchan;	/* sleep wchan */
	void		*t_exitstat;	/* exit status - non-detached threads */
	mutex_t		*t_handoff;	/* mutex hand off for cv_signal() */
	lwp_sema_t	t_park;		/* used to park threads */
	struct thread	*t_idle;	/* pointer to an idle thread */
	struct callo	t_alrm_callo;	/* alarm callout (per thread) */
	struct callo	t_cv_callo;	/* cv_timedwait callout (per thread) */
	struct itimerval t_realitimer;	/* real time interval timer */
	struct thread	*t_next;	/* circular queue of all threads */
	struct thread	*t_prev;
	/* only multiplexing threads use the following fields */
	mutex_t		t_lock;		/* locked when loaded into a LWP */
	struct fpu	t_fpu;		/* fpu stuff */
	struct thread	*t_iforw;	/* circular queue of idling threads */
	struct thread	*t_ibackw;
	struct thread	*t_forw;	/* circular queue of ONPROC threads */
	struct thread	*t_backw;
	int		t_errno;	/* thread specific errno */
} uthread_t;

#endif /* _LIBTHREAD_ */
