/*
 * @(#)longlong_md.h	1.00 97/02/25
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _UNIX_LONGLONG_H_
#define _UNIX_LONGLONG_H_

/*
 * ibm.2650: SB
 * Bodge the definitions of int32_t, int64_t, uint32_t, and uint64_t.
 * We avoid the symbol name clash by using #defines to 'trick' the
 * compiler into not using the AIX4.3 definitions of these symbols.
 */
#ifdef IBM_AIX        /*ibm.2650*/
#if defined(_AIX43) /* ibm.4491 */

#ifndef _LONG_LONG /* ibm.5473 */
#define _LONG_LONG /* ibm.5473 */
#undef __64BIT__   /* ibm.5473 */
#endif             /* ibm.5473 */

  /* Include the AIX4.3 definitions */
#include <inttypes.h>

#endif
#endif                /*ibm.2650*/

#include <stdint.h>

/*
 * ibmu02602
 * uint_t is defined here to avoid polluting Javasoft's typedefs_md.h
 * with platform specific stuff.
 * Posix defines it in sys/types.h, but it's missing from these platforms.
 */
#if defined(__MVS__) || defined(__VM__) || defined(IBM_LINUX)				/*ibmu02602*/
typedef unsigned int	uint_t;
#endif                                                          /*ibmu02602*/

/*
 * ibm01229
 * Allow use of this header independently of other float_t/double_t support.
 */
#ifndef	float_t
	#define	float_t	float
#endif

#ifndef	double_t
	#define double_t double
#endif

/*
 * ibm01229
 * Generic invocation of 32-bit int operations.
 */
#define INT_OP(x,op,y)  ( (x) op (y) )

/*
 * ibm01229
 * Provide indirection for long long emulation.
 */

#ifdef	NEEDSLONGLONG
#ifndef IBM_LINUX
        #define NEEDSPACKSTRUCT					/*ibmu02602*/

	/*
	 * Unsigned 64-bit integer declarations.
	 */

	#ifndef	_UINT64_T
		#define	_UINT64_T

		/*
		 * This is the first layer of the union used
		 * to represent an unsigned 64 bit integer.
		 * This allows static initialisation using:
		 *
		 *    uint64_t x = {0,0};
		 */
		
                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct 
			{
				unsigned long	hi;
				unsigned long	lo;
			} us1;
                #include <packoff_md.h>				/*ibmu02602*/

		/*
		 * This is the second layer of the union used
		 * to represent 64 bit integers. It identifies
		 * the various bits of interest when manipulating
		 * a simulated integer.
		 */

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned int	sh:1;
				unsigned int	vh:(sizeof(unsigned long)*8)-2;
				unsigned int	lh:1;
				unsigned int	sl:1;
				unsigned int	vl:(sizeof(unsigned long)*8)-2;
				unsigned int	ll:1;
			} us2;
                #include <packoff_md.h>				/*ibmu02602*/

		/*
		 * This is the third layer of the union used
		 * to represent 64 bit integers. It identifies
		 * the 4 short components used to perform 
		 * multiplication.
		 */

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned short	n4;
				unsigned short	n3;
				unsigned short	n2;
				unsigned short	n1;
			} us3;
                #include <packoff_md.h>				/*ibmu02602*/

		/*
		 * This is the fourth layer of the union used
		 * to represent 64 bit integers. It identifies
		 * the various product components needed by
		 * multiplication.
		 */

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned short	m3;
				unsigned long	m2;
				unsigned short	m1;
			} us4;
                #include <packoff_md.h>				/*ibmu02602*/

		/*
		 * This *is* the union used to represent 64 bit
		 * unsigned integers.
		 */

		typedef union
		{
			us1	s1;
			us2	s2;
			us3	s3;
			us4	s4;
		} uint64_t;
	#endif

#if defined(__MVS__)                            /*ibm.8212*/
	#ifndef __uint32_t
		#define __uint32_t 1
		typedef unsigned long uint32_t;
	#endif
#else /* !__MVS__ */                            /*ibm.8212*/
	#ifndef _UINT32_T
		#define _UINT32_T
		typedef unsigned long uint32_t;
	#endif
#endif /* !__MVS__ */                            /*ibm.8212*/

	/*
	 * Signed 64-bit integer declarations.
	 * There is some unnecessary replication, but this
	 * has been done for simplicity.
	 */

	#ifndef	_INT64_T
	#define	_INT64_T

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct 
			{
				  signed long	hi;
				unsigned long	lo;
			} ss1;
                #include <packoff_md.h>				/*ibmu02602*/

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned int	sh:1;
				unsigned int	vh:(sizeof(unsigned long)*8)-2;
				unsigned int	lh:1;
				unsigned int	sl:1;
				unsigned int	vl:(sizeof(unsigned long)*8)-2;
				unsigned int	ll:1;
			} ss2;
                #include <packoff_md.h>				/*ibmu02602*/

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned short	n4;
				unsigned short	n3;
				unsigned short	n2;
				unsigned short	n1;
			} ss3;
                #include <packoff_md.h>				/*ibmu02602*/

                #include <packon_md.h>				/*ibmu02602*/
		typedef	MVSPACK struct
			{
				unsigned short	m3;
				unsigned long	m2;
				unsigned short	m1;
			} ss4;
                #include <packoff_md.h>				/*ibmu02602*/

		typedef union
		{
			ss1	s1;
			ss2	s2;
			ss3	s3;
			ss4	s4;
		} int64_t;
	#endif	

	#ifndef _INT32_T
		#define _INT32_T
		typedef long int32_t;
	#endif
	/*
	 * Some quick-references for accessing the various
	 * portions of the emulator union.
	*/

	#define	HI64	s1.hi
	#define	LO64	s1.lo
	#define	SIGN64	s2.sh

	#define	SH64	s2.sh
	#define	SL64	s2.sl
	#define	LH64	s2.lh
	#define	LL64	s2.ll

	#define	N4		s3.n4
	#define	N3		s3.n3
	#define	N2		s3.n2
	#define	N1		s3.n1

	#define	M3		s4.m3
	#define	M2		s4.m2
	#define	M1		s4.m1

	#define	LL_DEF(a,b,c,d)	int64_t a = { (c), (d) }
	#define	LL_CON(ll,hi,lo) ( ll_or( ll_shl( int2ll(hi), 32 ) \
					, uint2ll(lo) ) )

	/*	unary operators		*/

	#define ll_high(a)	(a).HI64	/*ibm.4473*/
	#define ll_low(a)	(a).LO64	/*ibm.4473*/
	extern int64_t	ll_neg(int64_t a);
	extern int64_t	ll_not(int64_t a);

	/*	binary arithmetic operators	*/

	extern int64_t 	ll_add(int64_t a, int64_t b);
	extern int64_t 	ll_div(int64_t a, int64_t b);
	extern int64_t 	ll_mul(int64_t a, int64_t b);
	extern int64_t 	ll_sub(int64_t a, int64_t b)	;
	extern int64_t 	ll_rem(int64_t a, int64_t b);

	/*	binary logical operators	*/

	extern int64_t 	ll_and(int64_t a, int64_t b);
	extern int64_t 	ll_or(int64_t a, int64_t b);
	extern int64_t	ll_xor(int64_t a, int64_t b);

	/*	shift operators	*/

	extern int64_t	ll_ushr(int64_t a, int n);
	extern int64_t 	ll_shl(int64_t a, int n);
	extern int64_t 	ll_shr(int64_t a, int n);

	/* casting operators	*/

	extern int64_t 	int2ll(int a);
	extern int	ll2int(int64_t a);
	extern int64_t	uint2ll(unsigned int a);

	#ifndef	NEEDSIEEE754
		/*ibm01358*/
		#define flt2int(f)	((int32_t) (f))
		#define dbl2int(f)	((int32_t) (f))
		extern float_t	ll2float(int64_t a);
		extern double_t	ll2double(int64_t a);
		extern int64_t 	float2ll(float_t f);
		extern int64_t 	double2ll(double_t f);
	#endif

	/* comparison operators */

	extern int	ll_ltz(int64_t ll);
	extern int 	ll_gez(int64_t ll);
	extern int 	ll_eqz(int64_t a);
	extern int 	ll_eq(int64_t a,int64_t b);
	extern int 	ll_ne(int64_t a,int64_t b);
	extern int 	ll_ge(int64_t a,int64_t b);
	extern int 	ll_uge(int64_t a,int64_t b);
	extern int 	ll_le(int64_t a,int64_t b);
	extern int 	ll_lt(int64_t a,int64_t b);
	extern int 	ll_ult(int64_t a,int64_t b);
	extern int 	ll_gt(int64_t a,int64_t b);

/*
 *
 */
	extern int64_t 	ll_zero_const;
	extern int64_t 	ll_negzero_const;		/*ibm.4423*/
	extern int64_t	ll_one_const;
	extern int64_t	ll_negone_const;		/*ibm.4423*/
	extern int64_t 	ll_max_const;
	extern int64_t	ll_min_const;

/*
 *
 */
	extern char * ll2str(int64_t a, char *s, char *limit);
	extern int64_t ll_strtoll(char * a, void *v, int base);
#endif */ IBM_LINUX */
#else
	/*
	 * This part is mostly intact from the original typedefs_md.h.
	 */

#if !defined _AIX43 && !defined(IBM_LINUX) /* ibm.5473 */
	#ifndef	_UINT64_T
		#define	_UINT64_T
		typedef unsigned long long uint64_t;
	#endif	
#if defined(__MVS__)                            /*ibm.8212*/
	#ifndef __uint32_t
		#define __uint32_t 1
		typedef unsigned long uint32_t;
	#endif
#else /* !__MVS__ */                            /*ibm.8212*/
	#ifndef	_UINT32_T
		#define _UINT32_T
		typedef unsigned long uint32_t;
	#endif	
#endif /* !__MVS__ */                            /*ibm.8212*/
	#ifndef	_INT64_T
		#define	_INT64_T
		typedef long long int64_t;
	#endif
	#ifndef	_INT32_T
		#define _INT32_T
		typedef long int32_t;
	#endif
#endif /* ibm.5473 */

	#define	LL_DEF(a,b,c,d)	int64_t a = (b)
	#define	LL_CON(ll,hi,lo) ( ll )

	#define ll_high(a)	((long)((a)>>32))
	#define ll_low(a)	((unsigned long)(a))	/*ibm.6063*/
	#define ll_neg(a)	(-(a))
	#define ll_not(a)	(~(a))
	#define ll_add(a, b)	((a) + (b))
#if defined(IBM_IEEE_HSP)               /*ibm.9898*/
    #define ll_div(a, b)    (((a) == ll_min_const && (b) == ll_negone_const) ? \
                            (a) : ((a) / (b)))
#else /*!IBM_IEEE_HSP*/                 /*ibm.9898*/
	#define ll_div(a, b)	((a) / (b))
#endif /*IBM_IEEE_HSP*/                 /*ibm.9898*/
	#define ll_mul(a, b)	((a) * (b))
	#define ll_sub(a, b)	((a) - (b))
	#define ll_rem(a,b)	((a) % (b))
	#define ll_and(a, b)	((a) & (b))
	#define ll_or(a, b)	((a) | (b))
	#define ll_xor(a, b)	((a) ^ (b))
	#define ll_ushr(a, n)	((uint64_t)(a) >> (n))
	#define ll_shl(a, n)	((a) << (n))
	#define ll_shr(a, n)	((a) >> (n))
	#define int2ll(a)	((int64_t)(a))
	#define ll2int(a)	((int)(a))
	#define uint2ll(a)	((uint64_t)(unsigned long)(a))

	#define ll_ltz(ll)	((ll)<0)
	#define ll_gez(ll)	((ll)>=0)
	#define ll_eqz(a)	((a) == 0)
	#define ll_eq(a, b)	((a) == (b))
	#define ll_ne(a,b)	((a) != (b))
	#define ll_ge(a,b)	((a) >= (b))
	#define ll_uge(a,b)	(((unsigned long long)(a)) >= ((unsigned long long)(b)))
	#define ll_le(a,b)	((a) <= (b))
	#define ll_lt(a,b)	((a) < (b))
	#define ll_ult(a,b)	(((unsigned long long)(a)) < ((unsigned long long)(b)))
	#define ll_gt(a,b)	((a) > (b))
	#define ll_zero_const	((int64_t) 0)
	#define ll_negzero_const ((int64_t) -0)		/*ibm.4423*/
	#define ll_one_const	((int64_t) 1)
	#define ll_negone_const	((int64_t) -1)		/*ibm.4423*/
	#define ll_min_const	((int64_t) 0x8000000000000000LL)
	#define ll_max_const	((int64_t) 0x7fffffffffffffffLL)

	extern char * ll2str(int64_t a, char *s, char *limit);

	#define	ll_strtoll	strtoll

	#ifndef	NEEDSIEEE754
		/*ibm01358*/
		#define ll2float(a)	((float_t) (a))
		#define ll2double(a)	((double) (a))
		/*ibm.12687*/
		/* Note that these macros are now unsafe as they execute their arguments twice */
		#ifdef IBM_LINUX
		    #include <float_md.h>
		    #define flt2int(f)      ((f) >= MAX_INT ? MAX_INT : ((int32_t)(f)))
		    #define dbl2int(f)      ((f) >= MAX_INT ? MAX_INT : ((int32_t)(f)))
		    #define float2ll(f)     ((f) >= ll_max_const ? ll_max_const : ((int64_t)(f)))
		    #define double2ll(f)    ((f) >= ll_max_const ? ll_max_const : ((int64_t)(f)))
		#else
		    #define flt2int(f)	    ((int32_t) (f))
		    #define dbl2int(f)	    ((int32_t) (f))
		    #define float2ll(f)	    ((int64_t) (f))
		    #define double2ll(f)    ((int64_t) (f))
		#endif
	#endif

#endif	/* NEEDSLONGLONG */

#endif	/* _UNIX_LONGLONG_H_ */
