/*
 * @(#)packoff_md.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
/*
 * packoff_md.h
 *
 * Resets structure packing to the default.
 * Intended to be used with packon_md.h.
 *
 * Each platform appears to implement structure packing differently.
 * This file should provide the support for C and C++ variants for
 * all necessary platforms. The default is an error case which
 * causes a compilation failure so that those porting to the
 * new platform realise they need to add support in these files.
 *
 * For example:
 *
 * #include <packon_md.h>
 * MVSPACK struct tag {
 *    ...
 * } ;
 * #include <packoff_md.h>
 *
 * or
 *
 * #include <packon_md.h>
 * typedef MVSPACK struct {
 *    ...
 * } name ;
 * #include <packoff_md.h>
 *
 * 06Aug1997 ibmu02602 RL  original release
 */

#if defined(NEEDSPACKSTRUCT)

#if defined(__MVS__) || defined(__VM__)

    /***************************************************************
        OS390 (MVS) and VM Open Editions
    ****************************************************************/

    #if defined(__cplusplus)
        #pragma pack(reset)
    #endif

#else
#if defined(_AIX)

    /***************************************************************
        AIX
    ****************************************************************/

    #pragma options align=reset

#else
#if defined(_OS9000) || defined(__ILEC400__)

    /***************************************************************
        OS9000 and AS400
        nothing to do
    ****************************************************************/

#else

    /****************************************************************
        NEW PLATFORM
        Add explicit support after the preceding #else.
        Do not remove the following #error statement.
    ****************************************************************/

    #error __FILE__ : packoff_md.h unrecognized platform

#endif    /* _OS9000 or __ILEC400__ */
#endif    /* _AIX */
#endif    /* __MVS__ or __VM__ */

#endif /* NEEDSPACKSTRUCT */
