/*
 * @(#)sysmacros_md.h	1.3 96/03/20
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef _AIX_SYSMACROS_MD_H_
#define _AIX_SYSMACROS_MD_H_

/*
 * #define sysMalloc(size) (malloc(size))
 * #define sysFree(ptr)	(free(ptr))
 * #define sysCalloc(num, size) (calloc(num, size))
 * #define sysRealloc(ptr, size) (realloc(ptr, size))
 */

/*
 * because these are used directly as function ptrs, just redefine the name */

/*
** Stephen Blackheath  4 NOV 96:
**
** In src/shared/java/runtime/classresolver.c, in a function called
**
**   static char * ResolveInterfaces(ClassClass *cb)
**
** there is the following code:
**
**         for (i = super_itable_count; i < icount; i++) {
**            // The table length is the number of interface methods
**            ClassClass *intfi = this_itable->itable[i].classdescriptor;
**            int intfi_count = intfi->methods_count;
**            unsigned long *offsets =
**                sysMalloc(intfi_count * sizeof(unsigned long));
**            if (offsets == NULL) {
**                result = "OutOfMemoryError#3#";
**                goto failure;
**            }
**
** The problem with this is that 'intfi_count' can be zero, which means that
** zero is passed to sysMalloc.  If you just #define sysMalloc like this
**
**   #define sysMalloc malloc
**
** then this does not work, because malloc returns 0 if given a size of zero.
** It would appear that under Solaris, it works like the C++ specification of
** new, whereby if given a zero size, it returns a pointer which can be
** normally freed.  The caller should, of course, not store into this
** address.
**
** Given this assumption, I am changing sysMalloc to a simple function
** instead of a macro, so it can cope with this problem.
** See src/aix/java/include/sysmacros_md.h and
**     src/aix/java/runtime/sysmacros_md.c
*/

void* sysMalloc(size_t Size);
#ifdef NATIVE
#define sysFree		free
#define sysCalloc	calloc
#define sysRealloc	realloc
#else
void sysFree(void* mem);
void* sysCalloc(size_t elts, size_t size);
void* sysRealloc(void* mem, size_t size);
#endif

/* A macro for sneaking into a sys_mon_t to get the owner sys_thread_t */
#ifdef NATIVE
#define sysMonitorOwner(mid)   _sysThreadSelf((mid)->monitor_owner)
#else
#define sysMonitorOwner(mid)   (((void**)(mid))[0])
#endif

/* filesystem macros */

#ifdef DEBUG
extern void DumpThreads(void);
void panic (const char *, ...);
#define sysAssert(expression) {		\
    if (!(expression)) {		\
	DumpThreads();			\
	panic("\"%s\", line %d: assertion failure\n", __FILE__, __LINE__); \
    }					\
}
#else
#define sysAssert(expression) 0
#endif

/*
 * Check whether an exception occurred.  This also gives us the oppor-
 * tunity to use the already-required exception check as a trap for other
 * system-specific conditions.
 */
#define sysCheckException(ee) \
	if (!exceptionOccurred(ee)) { \
	   continue; \
	}

/*
 * Case insensitive compare of ASCII strings
 */
#define sysStricmp(a, b)		strcasecmp(a, b)

/*
 * file system macros moved here from io_md.h
 */
/* #define sysOpen(_path,_oflag,_mode)     open(_path,_oflag,_mode) */
#define sysRead(_fd,_buf,_n)		read(_fd,_buf,_n)
#define sysWrite(_fd,_buf,_n)		write(_fd,_buf,_n)
#define sysClose(_fd)			close(_fd)
#define sysAccess(_path,_mode)		access(_path,_mode)
#define sysStat(_path,_buf)		stat(_path,_buf)
#define sysMkdir(_path,_mode)		mkdir(_path,_mode)
#define sysUnlink(_path)		unlink(_path)
#define sysIsAbsolute(_path)		(*(_path) == '/')
#define sysCloseDir(_dir)		closedir(_dir)
#define sysOpenDir(_path)		opendir(_path)
#define sysRmdir(_dir)			remove(_dir)
#define sysNativePath(path)		(path)
#define sysSeek(fd, offset, whence)     lseek(fd, offset, whence)
#define sysRename(s, d)                 rename(s, d)

/*
 * Set up thread-local storage for second order monitor cache.  The
 * number of words of thread-local storage must be a power of 2!
 */
#define SYS_TLS_MONCACHE 8 /* must be power of 2 */
#define sysCurrentCacheKey(tid) (tid->cacheKey)
#define sysLocalMonCache(tid, hash) (tid->monitorCache[hash])

/*
 * Solaris always runs in Total Store Order mode for Sparc and
 * Intel doesn't seem to support a weak consistency model for MP
 * so we define this to do nothing. ALERT !! This needs to be investigated
 * for AIX.
 */
#define sysStoreBarrier() 0


/*
** ------ CACHE LOCKS -------------------------------------------------------
*/

#if defined(MODULE_javah) || defined(MODULE_javap)

  /* If this module is 'javah' then we don't want any code compiled
  ** for forbid() and permit()
  */
#define forbid()
#define permit()

#define sysCacheLockInit()
#define sysCacheLock()     forbid()
#define sysCacheUnlock()   permit()
#define sysCacheLocked()   1

/* ibm.5487 */
#else

#ifdef NATIVE

#include <pthread.h>
#include "monitor_md.h"

#ifndef MON_FLAT /* ibm.3740 */
/*ibm.4214: changed from a pthread mutex to a sysMonitor. */
extern struct sys_mon _monitor_cache_mon; /* ibm.4284 - struct sys_mon */

/* ibm.4284 - change _monitor_cache_lock to _monitor_cache_mon in the following */
#define sysCacheLockInit()      sysMonitorInit(&_monitor_cache_mon) /* ibm.4284 */
#define sysCacheLock()          sysMonitorEnter(&_monitor_cache_mon) /*ibm.4284 */
#define sysCacheUnlock()        sysMonitorExit(&_monitor_cache_mon) /* ibm.4282 */
#define sysCacheLocked()        1
#endif /* ibm.3740 */

  /* Forbid and permit do nothing in NATIVE threads implementation */
#define forbid()
#define permit()

/*#define sysMemoryFlush() 0					  ibm.6727 */
#define sysIsMP() 0

/* ibm.5487 */
/* ibm.6232  - don't need these anymore 
#define sysMonitorEnterQuicker(mid, tid) sysMutexLock(mid, tid)
#define sysMonitorExitQuicker(mid, tid) sysMutexUnlock(mid, tid)
*/

#else    /* NATIVE is not defined */

#define sysCacheLockInit()
#define sysCacheLock()     forbid()
#define sysCacheUnlock()   permit()
#define sysCacheLocked()   1

/*#define sysMemoryFlush() 0					  ibm.6727 */
#define sysIsMP() 0

#endif   /* NATIVE */
#endif   /* MODULE_javah */

/* Stephen Blackheath  15 Jan 97
** sysInterruptsPending() is so slow, that it is slowing garbage collection down
** - so - we make it do nothing to achieve a massive speed increase.
*/
#define sysInterruptsPending() 0
 
#ifdef JIT_WAY
extern volatile int threads_initialized;

#define sysThreadsInitialized()        threads_initialized
#endif /* JIT_WAY */

#ifndef IBM_LINUX
#undef GC_CLEAR_STACK /* ibm.4158 */
#define GC_CLEAR_STACK(tid)  sysGCClearStack(tid,1024) /* ibm.4158 *//*ibm.4292*/
#endif /* IBM_LINUX */
#endif /*_AIX_SYSMACROS_MD_H_*/
