/*
 * src/unix/java/native_threads/include/threads_md.h, aix, a110, a110-19970404 97/03/13
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

#ifndef POSIX_THREADS_MD_H
#define POSIX_THREADS_MD_H

#include        <pthread.h>
#include        <setjmp.h>
#include "sys_api.h"

#include <sys/user.h>	/* for PAGE_SIZE */
#define SYSSTACKUSED    (4096+MINSIGSTKSZ+PAGE_SIZE*2)
	/* PROCSTACKSIZE is expanded to 256KB for JIT_TRL, see javai.c */

#define CSTACKSIZE      PTHREAD_STACK_MIN+(4096*32)
#define ISTACKSIZE      PTHREAD_STACK_MIN

#define STARTING        1
#define READY           2
#define ZOMBIE          3
#define EXITING         4

#define STOPPED         11
#define SUSPEND         12

#define RUNNING         21
#define WAITING         22
#define SLEEPING        23
#define MON_WAIT        24
#define BLOCKING        25              /* Blocking on I/O */

/* Possible values for tid->suspend_request and tid->suspend_actual */
#define IC_NONE         0
#define IC_SUSPEND      1
#define IC_SINGLE       2

extern volatile int TQLock;
extern volatile int TQStack;
extern volatile int TQSingle;

#ifdef _AIX43_SUSPEND
/* Maximum registers to save during GC + spares.               ibm.f.6424 */
#define MAX_GC_REGS     40
#endif

/* usleep(1) here screws "string" benchmarks, but usleep(500) in sysMutexLock
   screws SMP performance! */
/* ibm.5906 - define YIELD according to no. of processors */
#define YIELD if(noOfProcessors==1) usleep(500); /* ibm.5989 */

#ifndef IBM_LINUX    /* PFE LINUX */
#define LOCK_SINGLE(a) \
        { \
        int Zero = 0; \
        while(!compare_and_swap((int*) &TQSingle,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}

#define LOCK_STACK(a) \
        { \
        int Zero = 0; \
        while(!compare_and_swap((int*) &TQStack,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}
/* ibm.5989 - use _check_lock instead of compare_and_swap
#define LOCK_QUEUE(a) \
        { \
        int Zero = 0; \
        while(!compare_and_swap((int*) &TQLock,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}
        */
#define LOCK_QUEUE(a) while(_check_lock((atomic_p)&TQLock,0,1)) YIELD


#define UNLOCK_SINGLE   TQSingle = 0
#define UNLOCK_STACK    TQStack = 0
/*  ibm.5989 - use _check_lock instead of compare_and_swap
#define UNLOCK_QUEUE    TQLock = 0 */
#define UNLOCK_QUEUE    _clear_lock((atomic_p)&TQLock,0)
#else
/* Linux definitions */
#define LOCK_SINGLE(a) \
        { \
        int Zero = 0; \
        while(!COMPARE_AND_SWAP((int*) &TQSingle,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}

#define LOCK_STACK(a) \
        { \
        int Zero = 0; \
        while(!COMPARE_AND_SWAP((int*) &TQStack,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}

#define LOCK_QUEUE(a) \
        { \
        int Zero = 0; \
        while(!COMPARE_AND_SWAP((int*) &TQLock,&Zero,(int)(1))) \
        { \
                YIELD; \
                Zero = 0; \
        }}

#define UNLOCK_SINGLE   TQSingle = 0
#define UNLOCK_STACK    TQStack = 0
#define UNLOCK_QUEUE    TQLock = 0
#endif /* PFE LINUX */

#ifdef  IBM_AIX
#define SIGSETMASK      sigthreadmask
#else
#define SIGSETMASK      sigprocmask
#endif

/*
** Definitions for thread_type:
*/

#define TT_USER         0       /* A user thread - once there are no user threads
                                ** the JVM terminates. */

#define TT_DAEMON       1       /* Daemon - JVM can terminate with daemon threads
                                ** still running, i.e., they are not counted in
                                ** the same way as user threads. */

#define TT_JNI          2       /* A JNI thread does not make use of the Zombie
                                ** functionality, so it cannot be re-used once it
                                ** has terminated.  JNI threads are considered
                                ** USER, not DAEMON. */

extern int noOfProcessors; /* ibm.5906 */

struct  sys_mon;
struct  wait_record;

struct  sys_thread {

    volatile struct sys_thread *    Next;
    volatile struct sys_thread *    Prev;
    
    pthread_t       threadself;
    void *          cookie;
    volatile void * stackptr;
    volatile void * cstack_frame;   /*ibm.3738*/
    void *          stackbase;
    long            stacksize;
    
    int             thread_type;
    volatile int    state;
    int             primordial_thread;
    int             usage;
    int             priority;
    volatile int    suspend_request;        /* requested suspend state */
    volatile int    suspend_actual;         /* actual suspend state */
    volatile int    in_pthread_critical_section;
    volatile int    in_blocking_section;
    volatile int    suspend_pending;
#ifdef MON_FLAT /* ibm.3740 */
    long index;
#endif /* ibm.3740 */
    volatile struct sys_mon* wait_monitor;  /* Monitor we're waiting on if waiting */
    volatile struct wait_record* wait;      /* This thread's wait record if it
					    ** is waiting on the above monitor. */
    volatile int    interrupted;    /* Feature of 1.1 */
    volatile int    wake_up_waiter;         /* ibm.3862: Flag to tell a waiting thread
					     * to wake up */
    
    void *          (*start)(void *);
    sigset_t        imask;
#ifdef MON_FLAT /* ibm.3813 */
    sys_mon_t *sleep_lock;
#endif /* ibm.3813 */
    
    pthread_mutex_t syncm;
    pthread_cond_t  syncc;
    
    sigjmp_buf      terminate_sigjmp;
    
    volatile void*  monitorObject;          /* The monitor object currently in use
					     * by this thread. Used in stack dump. */
    unsigned int cacheKey;
    void * monitorCache[SYS_TLS_MONCACHE];
#ifdef JIT_TRL
    volatile struct sigcontext *suspend_sigcontext; /* for fixed buffer */
#endif
#if defined(IBM_LINUX) && defined(JIT_TRL) /* kawatiya 1998/12/25 */
    void *jit_exception_handlers;	/* Link to SEHR_core structure */
#endif
#if defined(LINUX_LDT)	/* kawatiya 1999/03/10 */
    sys_thread_t *self_addr;		/* Pointer to self */
#endif
    volatile int in_finalization;
    
    /*       pthread_mutex_t localMutex; ibm.5487, 6232 */
    /* pthread_cond_t localCond;  ibm.5487, 6232 */
    /*     volatile struct sys_thread *   nextWaiter; ibm.5487, 6232 */
    /*     volatile struct sys_thread *   prevWaiter; ibm.5487, 6232 */
    volatile int blockLock; // ibm.12098 ibm.12369
//    pthread_mutex_t blockLock; // ibm.12098 ibm.12369
    int sysMonitorStatus;
    volatile sys_mon_t *sysMonitor;
    int currentFD;
    
#ifdef _AIX43_SUSPEND     /* ibm.f.6424  */
    /* This area is used to save context during GC.  It is updated by the
     * sysThreadSingle call.  Note that during a BlockingSection we DON'T
     * put the context here, it gets saved into a sigjmp_buf on the stack
     * and is thus traced naturally.  Hence we make this area look like a
     * vector and can control its length.  The nregs value says how many
     * of the vector elements are in use; during a system call we only need
     * to examine the non-volatile regs for instance.
     */
    long  nregs;
    int   gc_context[MAX_GC_REGS];
#endif                    /* ibm.f.6424  */
    volatile void * esp; /* ibm.11626 */
    pid_t	pid;						/* ibm.12094 */
};

#define enterPthreadCriticalSection(self) \
  self->in_pthread_critical_section = 1

struct sys_thread* _sysThreadSelf(pthread_t pid);
void doSuspendLoop(struct sys_thread* self);

#ifdef MON_FLAT /* ibm.3740 */
typedef struct thread_index_cb_t {
    long count;
    long length;
    sys_thread_t **buf;
} thread_index_cb_t;

extern thread_index_cb_t ticb;

#ifdef IBM_LINUX
extern int procfd;
#endif

sys_thread_t *
sysThreadIndexToThread (int idx);

long
sysThreadIndex (sys_thread_t *tid);

#endif /* ibm.3740 */

#define exitPthreadCriticalSection(self) \
  { \
    self->in_pthread_critical_section = 0; \
    if (self->suspend_pending) { \
      doSuspendLoop(self); \
      self->suspend_pending = 0; \
    } \
  }

#if defined(LINUX_LDT)	/* kawatiya 1999/03/11 */

#if 0 /* obsolete (kawatiya 1999/05/16) */
#define LDT_EXHANDLER	0	/* Entry 0 points jit_exception_handlers */
#define LDT_SYSTHREAD	1	/* Entry 1 points sys_thread_t */
#endif

/*
 * FS/GS-related inline assembler routines
 */
#define set_fs(seg) ({ register unsigned long __res; \
	__asm__ __volatile__("mov %%ax,%%fs":"=a" (__res):"0" (seg)); __res;})
#define get_fs() ({ register unsigned short __res; \
	__asm__("mov %%fs,%%ax":"=a" (__res):); __res;})
#define get_fs_mem(addr) ({ register unsigned long __res; \
	__asm__("movl %%fs:(%1),%%eax":"=a" (__res):"d" (addr)); __res;})
#define set_fs_mem(addr, val) ({ register unsigned long __res; \
	__asm__ __volatile__("movl %%eax,%%fs:(%1)": \
			"=a" (__res):"d" (addr), "0" (val)); __res;})
#define get_fs_0() ({ register unsigned long __res; \
	__asm__("movl %%fs:0,%%eax":"=a" (__res):); __res;})

#define set_gs(seg) ({ register unsigned long __res; \
	__asm__ __volatile__("mov %%ax,%%gs":"=a" (__res):"0" (seg)); __res;})
#define get_gs() ({ register unsigned short __res; \
	__asm__("mov %%gs,%%ax":"=a" (__res):); __res;})
#define get_gs_mem(addr) ({ register unsigned long __res; \
	__asm__("movl %%gs:(%1),%%eax":"=a" (__res):"d" (addr)); __res;})
#define set_gs_mem(addr, val) ({ register unsigned long __res; \
	__asm__ __volatile__("movl %%eax,%%gs:(%1)": \
			"=a" (__res):"d" (addr), "0" (val)); __res;})
#define get_gs_0() ({ register unsigned long __res; \
	__asm__("movl %%gs:0,%%eax":"=a" (__res):); __res;})

/*
 * NEEDSWORK: these inline assembler routines must be more sophisticated.
 *	Especially, I cannot find a way to generate "movl %fs:immaddr,%eax".
 *	The following cannot be compiled.
 *							kawatiya 1999/03/11
 */
#define get_fs_imm(immaddr) ({ register unsigned long __res; \
	__asm__("movl %%fs:(%0),%%eax":"=a" (__res):"i" (immaddr)); __res;})
#define get_gs_imm(immaddr) ({ register unsigned long __res; \
	__asm__("movl %%gs:(%0),%%eax":"=a" (__res):"i" (immaddr)); __res;})


#if 0 /* not used now (kawatiya 1999/05/16) */
/*
 * Fast access to a sys_thread_t member (4bytes) (GS points sys_thread_t)
 *	NEEDSWORK: it is better to use get_fs_imm()
 */
#include <stddef.h> /* for offsetof() macro */
#define sys_thread_get(member) \
		get_gs_mem(offsetof(sys_thread_t, member))
#define sys_thread_set(member, val) \
		set_gs_mem(offsetof(sys_thread_t, member), val)
#endif

#endif	/* LINUX_LDT */

#endif	/* !POSIX_THREADS_MD_H */
