/*
 * @(#)monitor_cache.h
 *
 * (c) Copyright Sun Microsystems Inc, 1992-1999. All rights reserved.
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 * 
 * This product is based in part on the work of the Independent JPEG group.
 * 
 * The Java(tm) technology is owned and exclusively licensed by 
 * Sun Microsystems Inc.  Java is a trademark of Sun Microsystems Inc. in
 * the U.S. and other countries.
 * 
 * US Govt Users Restricted Rights - Use duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
#ifdef IBM_FASTSWEEP		        /*ibm.6833*/

    #define     MARKABLE_MONITORS       /*ibm.6833 monitors support GC marking vs. sweeping */

#endif 					/*ibm.6833*/

/*
 * @(#)monitor_cache.h	1.21 97/10/07
 *
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 */

/*
 * Monitor cache definitions
 */

#ifndef _MONITOR_CACHE_H_
#define _MONITOR_CACHE_H_

#include "oobj.h"		/*ibm.6833*/
#include "monitor.h"		/* For TIMEOUT_INFINITY */

/*
 * There is a distinct recursive lock (not necessarily a monitor) for
 * the monitor cache.
 */
#ifdef MON_FLAT /* ibm.3740 */
extern sys_mon_t *_moncache_lock;
#define CACHE_LOCK_INIT() monitorRegister(_moncache_lock, "Monitor cache lock");
#define CACHE_LOCK()	  sysMonitorEnter(_moncache_lock)
#define CACHE_LOCKED()	  sysMonitorEntered(_moncache_lock)
#define CACHE_UNLOCK()	  sysMonitorExit(_moncache_lock)
#define CACHE_NOTIFY_ALL() sysMonitorNotifyAll(_moncache_lock)  /*ibm.4154*/
#define CACHE_WAIT()	  if (sysMonitorWait(_moncache_lock, TIMEOUT_INFINITY, FALSE) != 0) /*ibm.5642*/ \
				{ CACHE_UNLOCK(); sysThreadYield(); CACHE_LOCK(); }	    /*ibm.5642*/
#else
#define CACHE_LOCK_INIT() sysCacheLockInit()
#define CACHE_LOCK()	  sysCacheLock()
#define CACHE_LOCKED()	  sysCacheLocked()
#define CACHE_UNLOCK()	  sysCacheUnlock()
#endif /* ibm.3740 */

/*
 * External routines.
 */
#ifdef MON_FLAT /* ibm.3740 */
void allocMonitor(struct Hjava_lang_Object *);
void monitorDealloc(struct Hjava_lang_Object *);

#ifdef MARKABLE_MONITORS			/*ibm.6833*/
void clearMonitorMarks(void);			/*ibm.6833*/
void markMonitorForHandle(JHandle *);		/*ibm.6833*/
typedef unsigned char   monitor_index_mark_t;	/*ibm.6833*/
#endif /* MARKABLE_MONITORS */			/*ibm.6833*/

typedef struct monitor_index_cb_t {
  long count;
  long length;
  infl_mon_t **buf;
#ifdef MARKABLE_MONITORS			/*ibm.6833*/
  long marked;					/*ibm.6833*/
  monitor_index_mark_t *alloc;			/*ibm.6833*/
  monitor_index_mark_t *mark;			/*ibm.6833*/
#endif /* MARKABLE_MONITORS */			/*ibm.6833*/
} monitor_index_cb_t;

extern monitor_index_cb_t micb;

infl_mon_t *monitorIndexToMonitor(int idx);
int monitorIndex(infl_mon_t *mon);
#else
monitor_t *lookupMonitor(unsigned int);
monitor_t *createMonitor(unsigned int);
monitor_t *findMonitor(sys_thread_t *tid, unsigned int key, bool_t create);
void monitorEnumerate(void (*)(monitor_t *, void *), void *);
void monitorEnumerate_unlocked(void (*)(monitor_t *, void *), void *);

/*
 * This should probably just be defined as a function but it seems that
 * most of the x86 compilers aren't very agressive inliners and I'd
 * really like this to be inlined.
 */

#define checkCache(mon, tid, key) \
{ \
    extern int systemIsMP; \
    sys_thread_t * _tid = (tid); \
    unsigned _key = (key); \
    monitor_t * _mon; \
    if (tid) { \
        /* \
         * Stash the key currently being looked up; the monitor in the \
         * global cache for it, even if otherwise unused, is never collected. \
         */ \
        sysCurrentCacheKey(tid) = key; \
        if (systemIsMP) { \
            sysMemoryFlush(); \
        } \
        _mon = sysLocalMonCache(tid, (((key) >> 3) & (SYS_TLS_MONCACHE-1))); \
        if ((_mon != 0) && (_mon->key == key)) { \
            mon = _mon; \
        } else { \
            mon = NULL; \
        } \
    } else \
	mon = NULL; \
}

#endif

#endif /* !_MONITOR_CACHE_H_ */
